@@ -12,7 +12,6 @@ import time
 
 from xpra.log import Logger
 log = Logger()
-HOME = os.path.expanduser("~/")
 
 def getoutput(cmd, env=None):
     try:
@@ -26,160 +25,21 @@ def getoutput(cmd, env=None):
         raise Exception("command '%s' returned error code %s, out=%s, err=%s" % (cmd, code, out, err))
     return out
 
+def get_config(config_name):
+    try:
+        mod = __import__(config_name)
+    except (ImportError, SyntaxError) as e:
+        sys.stderr.write("Error loading module %s (%s)\n" % (config_name, e))
+        return None
+    return mod.Config()
 
-#You will probably need to change those:
-IP = "127.0.0.1"            #this is your IP
-PORT = 10000                #the port to test on
-DISPLAY_NO = 10             #the test DISPLAY no to use
-XORG_CONFIG="%s/xorg.conf" % HOME
-XORG_LOG = "%s/Xorg.%s.log" % (HOME, DISPLAY_NO)
-START_SERVER = True         #if False, you are responsible for starting it
-                            #and the data will not be available
-
-SETTLE_TIME = 3             #how long to wait before we start measuring
-MEASURE_TIME = 120          #run for N seconds
-COLLECT_STATS_TIME = 10     #collect statistics every N seconds
-SERVER_SETTLE_TIME = 3      #how long we wait for the server to start
-DEFAULT_TEST_COMMAND_SETTLE_TIME = 1    #how long we wait after starting the test command
-                            #this is the default value, some tests may override this below
-
-TEST_XPRA = True
-TEST_VNC = False            #WARNING: VNC not tested recently, probably needs updating
-USE_IPTABLES = False        #this requires iptables to be setup so we can use it for accounting
-USE_VIRTUALGL = True        #allows us to run GL games and benchmarks using the GPU
-PREVENT_SLEEP = True
-PREVENT_SLEEP_COMMAND = ["xdotool", "keydown", "Shift_L", "keyup", "Shift_L"]
-
-LIMIT_TESTS = 2
-LIMIT_TESTS = 99999         #to limit the total number of tests being run
-MAX_ERRORS = 100            #allow this many tests to cause errors before aborting
-
-#some commands (games especially) may need longer to startup:
-TEST_COMMAND_SETTLE_TIME = {}
-
-NO_SHAPING = (0, 0, 0)
-TRICKLE_SHAPING_OPTIONS = [NO_SHAPING]
-TRICKLE_SHAPING_OPTIONS = [NO_SHAPING, (1024, 1024, 20)]
-TRICKLE_SHAPING_OPTIONS = [(1024, 1024, 20), (128, 32, 40), (0, 0, 0)]
-TRICKLE_SHAPING_OPTIONS = [NO_SHAPING, (1024, 256, 20), (1024, 256, 300), (128, 32, 100), (32, 8, 200)]
-TRICKLE_SHAPING_OPTIONS = [NO_SHAPING, (1024, 256, 20), (256, 64, 50), (128, 32, 100), (32, 8, 200)]
-TRICKLE_SHAPING_OPTIONS = [NO_SHAPING]
-
-#tools we use:
-IPTABLES_CMD = ["sudo", "/usr/sbin/iptables"]
-TRICKLE_BIN = "/usr/bin/trickle"
-TCBENCH = "/opt/VirtualGL/bin/tcbench"
-TCBENCH_LOG = "./tcbench.log"
-XORG_BIN = "/usr/bin/Xorg"
-VGLRUN_BIN = "/usr/bin/vglrun"
-
-#the glx tests:
-GLX_SPHERES = ["/usr/bin/glxspheres"]
-GLX_GEARS = ["/usr/bin/glxgears", "-geometry", "1240x900"]
-GLX_TESTS = [GLX_SPHERES, GLX_GEARS]
-
-#the plain X11 tests:
-X11_PERF = ["/usr/bin/x11perf", "-resize", "-all"]
-XTERM_TEST = ["/usr/bin/xterm", "-geometry", "160x60", "-e", "while true; do dmesg; done"]
-FAKE_CONSOLE_USER_TEST = ["/usr/bin/xterm", "-geometry", "160x60", "-e", "PYTHONPATH=`pwd` ./tests/xpra/simulate_console_user.py"]
-GTKPERF_TEST = "bash -c 'while true; do gtkperf -a; done'"
-X11_TESTS = [X11_PERF, FAKE_CONSOLE_USER_TEST, GTKPERF_TEST]
-X11_TESTS = [X11_PERF, XTERM_TEST, FAKE_CONSOLE_USER_TEST, GTKPERF_TEST]
-
-#the screensaver tests:
-XSCREENSAVERS_PATH = "/usr/libexec/xscreensaver"
-def screensaver(x):
-    for d in [os.path.join(sys.prefix, "bin"), XSCREENSAVERS_PATH, "/usr/bin", "/usr/local/bin"]:
-        f = os.path.join(d, x)
-        if os.path.exists(f) and os.path.isfile(f):
-            return f
-    return  None
-ALL_SCREENSAVER_TESTS = [screensaver(x) for x in
-                            ["rss-glx-lattice", "rss-glx-plasma", "deluxe", "eruption", "memscroller", "moebiusgears", "polytopes"]
-                         ]
-SOME_SCREENSAVER_TESTS = [screensaver(x) for x in
-                            ["memscroller", "eruption", "xmatrix"]
-                          ]
-SOME_SCREENSAVER_TESTS = [screensaver(x) for x in
-                            ["memscroller", "moebiusgears", "polytopes", "rss-glx-lattice"]
-                          ]
-
-#games tests:
-#for more info, see here: http://dri.freedesktop.org/wiki/Benchmarking
-NEXUIZ_TEST = ["/usr/bin/nexuiz-glx", "-benchmark", "demos/demo1", "-nosound"]
-XONOTIC_TEST = ["/opt/Xonotic/xonotic-linux64-glx", "-basedir", "/opt/Xonotic", "-benchmark", "demos/the-big-keybench"]
-TEST_COMMAND_SETTLE_TIME[NEXUIZ_TEST[0]] = 10
-TEST_COMMAND_SETTLE_TIME[XONOTIC_TEST[0]] = 20
-GAMES_TESTS = [NEXUIZ_TEST, XONOTIC_TEST]
-
-#sound and video tests:
-VIDEO_TESTS = []
-SOUND_TESTS = []
-VLC_BIN = "/usr/bin/vlc"
-MPLAYER_BIN = "/usr/bin/mplayer"
-
-MPLAYER_SOUND_LOOP_TEST = "while true; do %s ./test.mp3; done" % MPLAYER_BIN
-VLC_SOUND_TEST = (VLC_BIN, "-L", "--audio-visual=visual", "./test.mp3")
-if not os.path.exists("test.mp3"):
-    print("test.mp3 not found, the corresponding sound mplayer sound and vlc video tests are disabled")
+if (len(sys.argv) > 1):
+    config_name = sys.argv[1]
 else:
-    SOUND_TESTS.append(MPLAYER_SOUND_LOOP_TEST)
-    VIDEO_TESTS.append(VLC_SOUND_TEST)
-
-#video tests
-VLC_VIDEO_TEST = (VLC_BIN, "-L", "./test.avi")
-MPLAYER_VIDEO_TEST = "while true; do %s test.avi; done" % MPLAYER_BIN
-if not os.path.exists("test.avi"):
-    print("test.avi not found, vlc and mplayer video tests are disabled")
-else:
-    VIDEO_TESTS.append(VLC_VIDEO_TEST)
-    VIDEO_TESTS.append(MPLAYER_VIDEO_TEST)
-
-#our selection:
-TEST_CANDIDATES = [screensaver("deluxe")]
-TEST_CANDIDATES = X11_TESTS + SOME_SCREENSAVER_TESTS + GAMES_TESTS
-TEST_CANDIDATES = GLX_TESTS + X11_TESTS + ALL_SCREENSAVER_TESTS + GAMES_TESTS
-TEST_CANDIDATES = GLX_TESTS + X11_TESTS + ALL_SCREENSAVER_TESTS + SOUND_TESTS + VIDEO_TESTS + GAMES_TESTS
-
-
-#now we filter all the test commands and only keep the valid ones:
-print("Checking for test commands:")
-X11_TEST_COMMANDS = []
-for x in TEST_CANDIDATES:
-    if x is None:
-        continue
-    if type(x) in (list, tuple) and not os.path.exists(x[0]):
-        print("* WARNING: cannot find %s - removed from tests" % str(x))
-    else:
-        print("* adding test: %s" % str(x))
-        X11_TEST_COMMANDS.append(x)
-TEST_NAMES = {GTKPERF_TEST: "gtkperf",
-              MPLAYER_SOUND_LOOP_TEST : "mplayer sound",
-              VLC_SOUND_TEST : "vlc sound visual",
-              MPLAYER_VIDEO_TEST : "mplayer video",
-              VLC_VIDEO_TEST : "vlc video",
-              }
-
-
-XVNC_BIN = "/usr/bin/Xvnc"
-XVNC_SERVER_START_COMMAND = [XVNC_BIN, "--rfbport=%s" % PORT,
-                   "+extension", "GLX",
-                   "--SecurityTypes=None",
-                   "--SendCutText=0", "--AcceptCutText=0", "--AcceptPointerEvents=0", "--AcceptKeyEvents=0",
-                   "-screen", "0", "1240x900x24",
-                   ":%s" % DISPLAY_NO]
-XVNC_SERVER_STOP_COMMANDS = [["killall Xvnc"]]     #stopped via kill - beware, this will kill *all* Xvnc sessions!
-VNCVIEWER_BIN = "/usr/bin/vncviewer"
-VNC_ENCODINGS = ["Tight", "ZRLE", "hextile", "raw", "auto"]
-VNC_ENCODINGS = ["auto"]
-VNC_ZLIB_OPTIONS = [-1, 3, 6, 9]
-VNC_ZLIB_OPTIONS = [-1, 9]
-VNC_COMPRESSION_OPTIONS = [0, 3, 8, 9]
-VNC_COMPRESSION_OPTIONS = [0, 3]
-VNC_JPEG_OPTIONS = [-1, 0, 8]
-VNC_JPEG_OPTIONS = [-1, 4]
-
-
+    config_name = 'perf_config_default'
+config = get_config(config_name)
+if (config==None):
+    raise Exception("Could not load config file")
 
 XPRA_BIN = "/usr/bin/xpra"
 XPRA_VERSION_OUTPUT = getoutput([XPRA_BIN, "--version"])
@@ -189,53 +49,28 @@ for x in XPRA_VERSION_OUTPUT.splitlines():
         XPRA_VERSION = x[len("xpra v"):].replace("\n", "").replace("\r", "")
 XPRA_VERSION_NO = [int(x) for x in XPRA_VERSION.split(".")]
 XPRA_SERVER_STOP_COMMANDS = [
-                             [XPRA_BIN, "stop", ":%s" % DISPLAY_NO],
-                             "ps -ef | grep -i [X]org-for-Xpra-:%s | awk '{print $2}' | xargs kill" % DISPLAY_NO
+                             [XPRA_BIN, "stop", ":%s" % config.DISPLAY_NO],
+                             "ps -ef | grep -i [X]org-for-Xpra-:%s | awk '{print $2}' | xargs kill" % config.DISPLAY_NO
                              ]
-XPRA_INFO_COMMAND = [XPRA_BIN, "info", "tcp:%s:%s" % (IP, PORT)]
-XPRA_FORCE_XDUMMY = False
-XPRA_QUALITY_OPTIONS = [40, 90]
-XPRA_QUALITY_OPTIONS = [80]
-XPRA_QUALITY_OPTIONS = [10, 40, 80, 90]
-XPRA_COMPRESSORS_OPTIONS = ["lz4", "zlib", "lzo", "zlib,lzo", "all", "none"]
-XPRA_COMPRESSORS_OPTIONS = ["all"]
-XPRA_COMPRESSION_LEVEL_OPTIONS = [0, 3, 9]
-XPRA_COMPRESSION_LEVEL_OPTIONS = [0, 3]
-XPRA_COMPRESSION_LEVEL_OPTIONS = [None]
-XPRA_PACKET_ENCODERS_OPTIONS = ["rencode", "bencode", "yaml"]
-XPRA_PACKET_ENCODERS_OPTIONS = ["rencode"]
-XPRA_CONNECT_OPTIONS = [("ssh", None), ("tcp", None), ("unix", None)]
-XPRA_CONNECT_OPTIONS = [("tcp", None)]
-#if XPRA_VERSION_NO>=[0, 7]:
-#    XPRA_CONNECT_OPTIONS.append(("tcp", "AES"))
+XPRA_INFO_COMMAND = [XPRA_BIN, "info", "tcp:%s:%s" % (config.IP, config.PORT)]
 print ("XPRA_VERSION_NO=%s" % XPRA_VERSION_NO)
-XPRA_TEST_ENCODINGS = ["png", "x264", "mmap"]
-XPRA_TEST_ENCODINGS = ["png", "jpeg", "x264", "vpx", "mmap"]
-XPRA_TEST_ENCODINGS = ["png", "rgb24", "jpeg", "x264", "vpx", "mmap"]
+
+STRICT_ENCODINGS = False
+if STRICT_ENCODINGS:
+    #beware: only enable this flag if the version being tested
+    # also supports the same environment overrides,
+    # or the comparison will not be fair.
+    os.environ["XPRA_ENCODING_STRICT_MODE"] = "1"
+    os.environ["XPRA_MAX_PIXELS_PREFER_RGB"] = "0"
+    os.environ["XPRA_MAX_NONVIDEO_PIXELS"] = "0"
+
 #webp leaks - don't test it:
 #if XPRA_VERSION_NO>=[0, 7]:
-#    XPRA_TEST_ENCODINGS.append("webp")
-XPRA_ENCODING_QUALITY_OPTIONS = {"jpeg" : XPRA_QUALITY_OPTIONS,
-                                 "webp" : XPRA_QUALITY_OPTIONS,
-                                 "x264" : XPRA_QUALITY_OPTIONS+[-1],
-                                 }
-XPRA_ENCODING_QUALITY_OPTIONS = {"jpeg" : [-1],
-                                 "x264" : [-1],
-                                 }
-XPRA_ENCODING_SPEED_OPTIONS = {
-                               "rgb24" : [-1, 0, 100],
-                               }
-XPRA_OPENGL_OPTIONS = {"x264" : [True, False],
-                       "vpx" : [True, False] }
-#only test default opengl setting:
-XPRA_OPENGL_OPTIONS = {}
-XPRA_MDNS = False
-
+#    config.XPRA_TEST_ENCODINGS.append("webp")
 
 XPRA_SPEAKER_OPTIONS = [None]
 XPRA_MICROPHONE_OPTIONS = [None]
-TEST_SOUND = False
-if TEST_SOUND:
+if config.TEST_SOUND:
     from xpra.sound.gstreamer_util import CODEC_ORDER, has_codec
     if XPRA_VERSION_NO>=[0, 9]:
         #0.9 onwards supports all codecs defined:
@@ -247,29 +82,22 @@ if TEST_SOUND:
         #none before that
         XPRA_SPEAKER_OPTIONS = [None]
 
-
-XPRA_USE_PASSWORD = True
-password_filename = "./test-password.txt"
-try:
+if (config.XPRA_USE_PASSWORD):
+    password_filename = "./test-password.txt"
     import uuid
-    f = open(password_filename, 'wb')
-    f.write(uuid.uuid4().hex)
-finally:
-    f.close()
+    with open(password_filename, 'wb') as f:
+        f.write(uuid.uuid4().hex)
 
-
-check = [TRICKLE_BIN]
-if TEST_XPRA:
+check = [config.TRICKLE_BIN]
+if config.TEST_XPRA:
     check.append(XPRA_BIN)
-if TEST_VNC:
-    check.append(XVNC_BIN)
-    check.append(VNCVIEWER_BIN)
+if config.TEST_VNC:
+    check.append(config.XVNC_BIN)
+    check.append(config.VNCVIEWER_BIN)
 for x in check:
     if not os.path.exists(x):
         raise Exception("cannot run tests: %s is missing!" % x)
 
-
-
 HEADERS = ["Test Name", "Remoting Tech", "Server Version", "Client Version", "Custom Params", "SVN Version",
            "Encoding", "Quality", "Speed","OpenGL", "Test Command", "Sample Duration (s)", "Sample Time (epoch)",
            "CPU info", "Platform", "Kernel Version", "Xorg version", "OpenGL", "Client Window Manager", "Screen Size",
@@ -290,7 +118,6 @@ for h in ("Batch Delay (ms)", "Actual Batch Delay (ms)",
     for x in ("Min", "Avg", "Max"):
         HEADERS.append(x+" "+h)
 
-
 def is_process_alive(process, grace=0):
     i = 0
     while i<grace:
@@ -347,7 +174,7 @@ def get_cpu_info():
     print("CPU_INFO=%s" % cpu_info)
     return  cpu_info, n
 
-XORG_VERSION = getoutput_line([XORG_BIN, "-version"], "X.Org X Server", "Cannot detect Xorg server version")
+XORG_VERSION = getoutput_line([config.XORG_BIN, "-version"], "X.Org X Server", "Cannot detect Xorg server version")
 print("XORG_VERSION=%s" % XORG_VERSION)
 CPU_INFO, N_CPUS = get_cpu_info()
 KERNEL_VERSION = getoutput(["uname", "-r"]).replace("\n", "").replace("\r", "")
@@ -365,8 +192,8 @@ print("screen size=%s" % str(SCREEN_SIZE))
 #detect Xvnc version:
 XVNC_VERSION = ""
 VNCVIEWER_VERSION = ""
-DETECT_XVNC_VERSION_CMD = [XVNC_BIN, "--help"]
-DETECT_VNCVIEWER_VERSION_CMD = [VNCVIEWER_BIN, "--help"]
+DETECT_XVNC_VERSION_CMD = [config.XVNC_BIN, "--help"]
+DETECT_VNCVIEWER_VERSION_CMD = [config.VNCVIEWER_BIN, "--help"]
 def get_stderr(command):
     try:
         process = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -416,9 +243,9 @@ def clean_sys_state():
     assert process.wait()==0, "failed to run %s" % str(cmd)
 
 def zero_iptables():
-    if not USE_IPTABLES:
+    if not config.USE_IPTABLES:
         return
-    cmds = [IPTABLES_CMD+['-Z', 'INPUT'], IPTABLES_CMD+['-Z', 'OUTPUT']]
+    cmds = [config.IPTABLES_CMD+['-Z', 'INPUT'], config.IPTABLES_CMD+['-Z', 'OUTPUT']]
     for cmd in cmds:
         getoutput(cmd)
         #out = getoutput(cmd)
@@ -465,14 +292,14 @@ def compute_stat(prefix, time_total_diff, old_pid_stat, new_pid_stat):
             }
 
 def getiptables_line(chain, pattern, setup_info):
-    cmd = IPTABLES_CMD + ["-vnL", chain]
+    cmd = config.IPTABLES_CMD + ["-vnL", chain]
     line = getoutput_line(cmd, pattern, setup_info)
     if not line:
         raise Exception("no line found matching %s, make sure you have a rule like: %s" % (pattern, setup_info))
     return line
 
 def parse_ipt(chain, pattern, setup_info):
-    if not USE_IPTABLES:
+    if not config.USE_IPTABLES:
         return  0, 0
     line = getiptables_line(chain, pattern, setup_info)
     parts = line.split()
@@ -481,24 +308,23 @@ def parse_ipt(chain, pattern, setup_info):
         U = 1024
         m = {"K":U, "M":U**2, "G":U**3}.get(part[-1], 1)
         num = "".join([x for x in part if x in "0123456789"])
-        return int(num)*m/MEASURE_TIME
+        return int(num)*m/config.MEASURE_TIME
     return parse_num(parts[0]), parse_num(parts[1])
 
 def get_iptables_INPUT_count():
-    setup = "iptables -I INPUT -p tcp --dport %s -j ACCEPT" % PORT
-    return  parse_ipt("INPUT", "tcp dpt:%s" % PORT, setup)
+    setup = "iptables -I INPUT -p tcp --dport %s -j ACCEPT" % config.PORT
+    return  parse_ipt("INPUT", "tcp dpt:%s" % config.PORT, setup)
 
 def get_iptables_OUTPUT_count():
-    setup = "iptables -I OUTPUT -p tcp --sport %s -j ACCEPT" % PORT
-    return  parse_ipt("OUTPUT", "tcp spt:%s" % PORT, setup)
-
+    setup = "iptables -I OUTPUT -p tcp --sport %s -j ACCEPT" % config.PORT
+    return  parse_ipt("OUTPUT", "tcp spt:%s" % config.PORT, setup)
 
 def measure_client(server_pid, name, cmd, get_stats_cb):
     print("starting client: %s" % cmd)
     try:
         client_process = subprocess.Popen(cmd)
         #give it time to settle down:
-        time.sleep(SETTLE_TIME)
+        time.sleep(config.SETTLE_TIME)
         code = client_process.poll()
         assert code is None, "client failed to start, return code is %s" % code
         #clear counters
@@ -511,9 +337,9 @@ def measure_client(server_pid, name, cmd, get_stats_cb):
         #we start measuring
         t = 0
         all_stats = [initial_stats]
-        while t<MEASURE_TIME:
-            time.sleep(COLLECT_STATS_TIME)
-            t += COLLECT_STATS_TIME
+        while t<config.MEASURE_TIME:
+            time.sleep(config.COLLECT_STATS_TIME)
+            t += config.COLLECT_STATS_TIME
 
             code = client_process.poll()
             assert code is None, "client crashed, return code is %s" % code
@@ -539,7 +365,7 @@ def measure_client(server_pid, name, cmd, get_stats_cb):
         else:
             server_process_data = []
         print("process_data (client/server): %s / %s" % (client_process_data, server_process_data))
-        print("input/output on tcp port %s: %s / %s packets, %s / %s KBytes" % (PORT, ni, no, isize, osize))
+        print("input/output on tcp PORT %s: %s / %s packets, %s / %s KBytes" % (config.PORT, ni, no, isize, osize))
         data = {}
         data.update(iptables_stat)
         data.update(stats)
@@ -555,10 +381,10 @@ def measure_client(server_pid, name, cmd, get_stats_cb):
             assert code is not None, "failed to stop client!"
 
 def with_server(start_server_command, stop_server_commands, in_tests, get_stats_cb):
-    tests = in_tests[:LIMIT_TESTS]
+    tests = in_tests[config.STARTING_TEST:config.LIMIT_TESTS]
     print("going to run %s tests: %s" % (len(tests), [x[0] for x in tests]))
     print("*******************************************")
-    print("ETA: %s minutes" % int((SERVER_SETTLE_TIME+DEFAULT_TEST_COMMAND_SETTLE_TIME+SETTLE_TIME+MEASURE_TIME+1)*len(tests)/60))
+    print("ETA: %s minutes" % int((config.SERVER_SETTLE_TIME+config.DEFAULT_TEST_COMMAND_SETTLE_TIME+config.SETTLE_TIME+config.MEASURE_TIME+1)*len(tests)/60))
     print("*******************************************")
 
     server_process = None
@@ -568,7 +394,7 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
     #whitelist what we want to keep:
         if k.startswith("XPRA") or k in ("LOGNAME", "XDG_RUNTIME_DIR", "USER", "HOME", "PATH", "LD_LIBRARY_PATH", "XAUTHORITY", "SHELL", "TERM", "USERNAME", "HOSTNAME", "PWD"):
             env[k] = v
-    env["DISPLAY"] = ":%s" % DISPLAY_NO
+    env["DISPLAY"] = ":%s" % config.DISPLAY_NO
     errors = 0
     results = []
     count = 0
@@ -577,18 +403,18 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
         try:
             print("**************************************************************")
             count += 1
-            test_command_settle_time = TEST_COMMAND_SETTLE_TIME.get(test_command[0], DEFAULT_TEST_COMMAND_SETTLE_TIME)
-            eta = int((SERVER_SETTLE_TIME+test_command_settle_time+SETTLE_TIME+MEASURE_TIME+1)*(len(tests)-count)/60)
+            test_command_settle_time = config.TEST_COMMAND_SETTLE_TIME.get(test_command[0], config.DEFAULT_TEST_COMMAND_SETTLE_TIME)
+            eta = int((config.SERVER_SETTLE_TIME+test_command_settle_time+config.SETTLE_TIME+config.MEASURE_TIME+1)*(len(tests)-count)/60)
             print("%s/%s: %s            ETA=%s minutes" % (count, len(tests), name, eta))
             test_command_process = None
             try:
                 clean_sys_state()
                 #start the server:
-                if START_SERVER:
+                if config.START_SERVER:
                     print("starting server: %s" % str(start_server_command))
                     server_process = subprocess.Popen(start_server_command, stdin=None)
                     #give it time to settle down:
-                    t = SERVER_SETTLE_TIME
+                    t = config.SERVER_SETTLE_TIME
                     if count==1:
                         #first run, give it enough time to cleanup the socket
                         t += 5
@@ -601,11 +427,11 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
 
                 try:
                     #start the test command:
-                    if USE_VIRTUALGL:
+                    if config.USE_VIRTUALGL:
                         if type(test_command)==str:
-                            cmd = VGLRUN_BIN + " -- "+ test_command
+                            cmd = config.VGLRUN_BIN + " -- "+ test_command
                         elif type(test_command) in (list, tuple):
-                            cmd = [VGLRUN_BIN, "--"] + list(test_command)
+                            cmd = [config.VGLRUN_BIN, "--"] + list(test_command)
                         else:
                             raise Exception("invalid test command type: %s for %s" % (type(test_command), test_command))
                     else:
@@ -615,8 +441,8 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                     shell = type(cmd)==str
                     test_command_process = subprocess.Popen(cmd, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env, shell=shell)
 
-                    if PREVENT_SLEEP:
-                        subprocess.Popen(PREVENT_SLEEP_COMMAND)
+                    if config.PREVENT_SLEEP:
+                        subprocess.Popen(config.PREVENT_SLEEP_COMMAND)
 
                     time.sleep(test_command_settle_time)
                     code = test_command_process.poll()
@@ -628,14 +454,14 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                             "Remoting Tech"  : tech_name,
                             "Server Version" : server_version,
                             "Client Version" : client_version,
-                            "Custom Params"  : " ".join(sys.argv[1:]),
+                            "Custom Params"  : config.CUSTOM_PARAMS,
                             "SVN Version"    : SVN_VERSION,
                             "Encoding"       : encoding,
                             "Quality"        : quality,
                             "Speed"          : speed,
                             "OpenGL"         : opengl,
                             "Test Command"   : get_command_name(test_command),
-                            "Sample Duration (s)"    : MEASURE_TIME,
+                            "Sample Duration (s)"    : config.MEASURE_TIME,
                             "Sample Time (epoch)"    : time.time(),
                             "CPU info"       : CPU_INFO,
                             "Platform"       : PLATFORM,
@@ -658,7 +484,7 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                     traceback.print_exc()
                     errors += 1
                     print("error during client command run for %s: %s" % (name, e))
-                    if errors>MAX_ERRORS:
+                    if errors>config.MAX_ERRORS:
                         print("too many errors, aborting tests")
                         break
             finally:
@@ -666,7 +492,7 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                     print("stopping '%s' with pid=%s" % (test_command, test_command_process.pid))
                     try_to_stop(test_command_process)
                     try_to_kill(test_command_process, 2)
-                if START_SERVER:
+                if config.START_SERVER:
                     try_to_stop(server_process)
                     time.sleep(2)
                     for s in stop_server_commands:
@@ -683,11 +509,10 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
             break
     return results
 
-
 def trickle_command(down, up, latency):
     if down<=0 and up<=0 and latency<=0:
         return  []
-    cmd = [TRICKLE_BIN, "-s"]
+    cmd = [config.TRICKLE_BIN, "-s"]
     if down>0:
         cmd += ["-d", str(down)]
     if up>0:
@@ -704,7 +529,7 @@ def trickle_str(down, up, latency):
 
 def get_command_name(command_arg):
     try:
-        name = TEST_NAMES.get(command_arg)
+        name = config.TEST_NAMES.get(command_arg)
         if name:
             return  name
     except:
@@ -716,12 +541,11 @@ def get_command_name(command_arg):
     assert type(c)==str
     return c.split("/")[-1]             #/usr/bin/xterm -> xterm
 
-
 def xpra_get_stats(initial_stats=None, all_stats=[]):
     if XPRA_VERSION_NO<[0, 3]:
         return  {}
     info_cmd = XPRA_INFO_COMMAND[:]
-    if XPRA_USE_PASSWORD and password_filename:
+    if config.XPRA_USE_PASSWORD:
         info_cmd.append("--password-file=%s" % password_filename)
     out = getoutput(info_cmd)
     if not out:
@@ -749,11 +573,11 @@ def xpra_get_stats(initial_stats=None, all_stats=[]):
     initial_output_bytecount   = lookup.get("Application bytes out/s", 0)
     initial_mmap_bytes         = lookup.get("mmap bytes/s", 0)
     data = {
-            "Application packets in/s"      : (iget(["client.connection.input.packetcount", "input_packetcount"], 0)-initial_input_packetcount)/MEASURE_TIME,
-            "Application bytes in/s"        : (iget(["client.connection.input.bytecount", "input_bytecount"], 0)-initial_input_bytecount)/MEASURE_TIME,
-            "Application packets out/s"     : (iget(["client.connection.output.packetcount", "output_packetcount"], 0)-initial_output_packetcount)/MEASURE_TIME,
-            "Application bytes out/s"       : (iget(["client.connection.output.bytecount", "output_bytecount"], 0)-initial_output_bytecount)/MEASURE_TIME,
-            "mmap bytes/s"                  : (iget(["client.connection.output.mmap_bytecount", "output_mmap_bytecount"], 0)-initial_mmap_bytes)/MEASURE_TIME,
+            "Application packets in/s"      : (iget(["client.connection.input.packetcount", "input_packetcount"], 0)-initial_input_packetcount)/config.MEASURE_TIME,
+            "Application bytes in/s"        : (iget(["client.connection.input.bytecount", "input_bytecount"], 0)-initial_input_bytecount)/config.MEASURE_TIME,
+            "Application packets out/s"     : (iget(["client.connection.output.packetcount", "output_packetcount"], 0)-initial_output_packetcount)/config.MEASURE_TIME,
+            "Application bytes out/s"       : (iget(["client.connection.output.bytecount", "output_bytecount"], 0)-initial_output_bytecount)/config.MEASURE_TIME,
+            "mmap bytes/s"                  : (iget(["client.connection.output.mmap_bytecount", "output_mmap_bytecount"], 0)-initial_mmap_bytes)/config.MEASURE_TIME,
             }
 
     #values that are averages or min/max:
@@ -836,15 +660,15 @@ def xpra_get_stats(initial_stats=None, all_stats=[]):
     return data
 
 def get_xpra_start_server_command():
-    cmd = [XPRA_BIN, "--no-daemon", "--bind-tcp=0.0.0.0:%s" % PORT]
-    if XPRA_FORCE_XDUMMY:
-        cmd.append("--xvfb=%s -nolisten tcp +extension GLX +extension RANDR +extension RENDER -logfile %s -config %s" % (XORG_BIN, XORG_LOG, XORG_CONFIG))
+    cmd = [XPRA_BIN, "--no-daemon", "--bind-tcp=0.0.0.0:%s" % config.PORT]
+    if config.XPRA_FORCE_XDUMMY:
+        cmd.append("--xvfb=%s -nolisten tcp +extension GLX +extension RANDR +extension RENDER -logfile %s -config %s" % (config.XORG_BIN, config.XORG_LOG, config.XORG_CONFIG))
     if XPRA_VERSION_NO>=[0, 5]:
         cmd.append("--no-notifications")
-    if XPRA_USE_PASSWORD:
+    if config.XPRA_USE_PASSWORD:
         cmd.append("--password-file=%s" % password_filename)
     cmd.append("--no-pulseaudio")
-    cmd += ["start", ":%s" % DISPLAY_NO]
+    cmd += ["start", ":%s" % config.DISPLAY_NO]
     return cmd
 
 def test_xpra():
@@ -853,46 +677,47 @@ def test_xpra():
     print("                Xpra tests")
     print("")
     tests = []
-    for connect_option, encryption in XPRA_CONNECT_OPTIONS:
-        shaping_options = TRICKLE_SHAPING_OPTIONS
+    for connect_option, encryption in config.XPRA_CONNECT_OPTIONS:
+        shaping_options = config.TRICKLE_SHAPING_OPTIONS
         if connect_option=="unix":
-            shaping_options = [NO_SHAPING]
+            shaping_options = [config.NO_SHAPING]
         for down,up,latency in shaping_options:
-            for x11_test_command in X11_TEST_COMMANDS:
-                for encoding in XPRA_TEST_ENCODINGS:
+            for x11_test_command in config.X11_TEST_COMMANDS:
+                for encoding in config.XPRA_TEST_ENCODINGS:
                     if XPRA_VERSION_NO>=[0, 10]:
-                        opengl_options = XPRA_OPENGL_OPTIONS.get(encoding, [True])
+                        opengl_options = config.XPRA_OPENGL_OPTIONS.get(encoding, [True])
                     elif XPRA_VERSION_NO>=[0, 9]:
-                        opengl_options = XPRA_OPENGL_OPTIONS.get(encoding, [False])
+                        opengl_options = config.XPRA_OPENGL_OPTIONS.get(encoding, [False])
                     else:
                         opengl_options = [False]
                     for opengl in opengl_options:
-                        quality_options = XPRA_ENCODING_QUALITY_OPTIONS.get(encoding, [-1])
+                        quality_options = config.XPRA_ENCODING_QUALITY_OPTIONS.get(encoding, [-1])
                         for quality in quality_options:
-                            speed_options = XPRA_ENCODING_SPEED_OPTIONS.get(encoding, [-1])
+                            speed_options = config.XPRA_ENCODING_SPEED_OPTIONS.get(encoding, [-1])
                             for speed in speed_options:
                                 for speaker in XPRA_SPEAKER_OPTIONS:
                                     for mic in XPRA_MICROPHONE_OPTIONS:
                                         comp_options = []
                                         if XPRA_VERSION_NO>=[0, 14]:
-                                            comp_options = XPRA_COMPRESSORS_OPTIONS
+                                            comp_options = config.XPRA_COMPRESSORS_OPTIONS
                                         for comp in comp_options:
-                                            comp_level_options = XPRA_COMPRESSION_LEVEL_OPTIONS
+                                            comp_level_options = config.XPRA_COMPRESSION_LEVEL_OPTIONS
                                             for compression in comp_level_options:
                                                 packet_encoders_options = []
                                                 if XPRA_VERSION_NO>=[0, 14]:
-                                                    packet_encoders_options = XPRA_PACKET_ENCODERS_OPTIONS
+                                                    packet_encoders_options = config.XPRA_PACKET_ENCODERS_OPTIONS
                                                 for packet_encoders in packet_encoders_options:
                                                     cmd = trickle_command(down, up, latency)
                                                     cmd += [XPRA_BIN, "attach"]
                                                     if connect_option=="ssh":
-                                                        cmd.append("ssh:%s:%s" % (IP, DISPLAY_NO))
+                                                        cmd.append("ssh:%s:%s" % (config.IP, config.DISPLAY_NO))
                                                     elif connect_option=="tcp":
-                                                        cmd.append("tcp:%s:%s" % (IP, PORT))
+                                                        cmd.append("tcp:%s:%s" % (config.IP, config.PORT))
                                                     else:
-                                                        cmd.append(":%s" % (DISPLAY_NO))
+                                                        cmd.append(":%s" % (config.DISPLAY_NO))
                                                     cmd.append("--readonly")
-                                                    cmd.append("--password-file=%s" % password_filename)
+                                                    if config.XPRA_USE_PASSWORD:
+                                                        cmd.append("--password-file=%s" % password_filename)
                                                     if packet_encoders:
                                                         cmd += ["--packet-encoders=%s" % packet_encoders]
                                                     if comp:
@@ -907,7 +732,7 @@ def test_xpra():
                                                         cmd.append("--no-cursors")
                                                         cmd.append("--no-notifications")
                                                     if XPRA_VERSION_NO>=[0, 12]:
-                                                        if XPRA_MDNS:
+                                                        if config.XPRA_MDNS:
                                                             cmd.append("--mdns")
                                                         else:
                                                             cmd.append("--no-mdns")
@@ -947,7 +772,6 @@ def test_xpra():
                                                                   (down,up,latency), x11_test_command, cmd))
     return with_server(get_xpra_start_server_command(), XPRA_SERVER_STOP_COMMANDS, tests, xpra_get_stats)
 
-
 def get_x11_client_window_info(display, *app_name_strings):
     env = os.environ.copy()
     if display:
@@ -991,7 +815,7 @@ def get_vnc_stats(initial_stats=None, all_stats=[]):
         #we first need to figure out the dimensions of the client window
         #within the Xvnc server, the use those dimensions to tell tcbench
         #where to look in the vncviewer client window
-        test_window_info = get_x11_client_window_info(":%s" % DISPLAY_NO)
+        test_window_info = get_x11_client_window_info(":%s" % config.DISPLAY_NO)
         print("info for client test window: %s" % str(test_window_info))
         info = get_x11_client_window_info(None, "TigerVNC: x11", "Vncviewer")
         if not info:
@@ -1002,15 +826,15 @@ def get_vnc_stats(initial_stats=None, all_stats=[]):
             return  {}
         if test_window_info:
             _, _, _, w, h = test_window_info
-        command = [TCBENCH, "-wh%s" % wid, "-t%s" % (MEASURE_TIME-5)]
+        command = [config.TCBENCH, "-wh%s" % wid, "-t%s" % (config.MEASURE_TIME-5)]
         if w>0 and h>0:
             command.append("-x%s" % int(w/2))
             command.append("-y%s" % int(h/2))
-        if os.path.exists(TCBENCH_LOG):
-            os.unlink(TCBENCH_LOG)
-        tcbench_log  = open(TCBENCH_LOG, 'w')
+        if os.path.exists(config.TCBENCH_LOG):
+            os.unlink(config.TCBENCH_LOG)
+        tcbench_log  = open(config.TCBENCH_LOG, 'w')
         try:
-            print("tcbench starting: %s, logging to %s" % (command, TCBENCH_LOG))
+            print("tcbench starting: %s, logging to %s" % (command, config.TCBENCH_LOG))
             proc = subprocess.Popen(command, stdin=None, stdout=tcbench_log, stderr=tcbench_log)
             return {"tcbench" : proc}
         except Exception, e:
@@ -1029,9 +853,8 @@ def get_vnc_stats(initial_stats=None, all_stats=[]):
             try_to_stop(process)
             try_to_kill(process, 2)
         else:
-            f = open(TCBENCH_LOG, mode='rb')
-            out = f.read()
-            f.close()
+            with open(config.TCBENCH_LOG, mode='rb') as f:
+                out = f.read()
             #print("get_vnc_stats(%s) tcbench output=%s" % (last_record, out))
             for line in out.splitlines():
                 if not line.find("Frames/sec:")>=0:
@@ -1049,17 +872,17 @@ def test_vnc():
     print("                VNC tests")
     print("")
     tests = []
-    for down,up,latency in TRICKLE_SHAPING_OPTIONS:
-        for x11_test_command in X11_TEST_COMMANDS:
-            for encoding in VNC_ENCODINGS:
-                for zlib in VNC_ZLIB_OPTIONS:
-                    for compression in VNC_COMPRESSION_OPTIONS:
+    for down,up,latency in config.TRICKLE_SHAPING_OPTIONS:
+        for x11_test_command in config.X11_TEST_COMMANDS:
+            for encoding in config.VNC_ENCODINGS:
+                for zlib in config.VNC_ZLIB_OPTIONS:
+                    for compression in config.VNC_COMPRESSION_OPTIONS:
                         jpeg_quality = [8]
                         if encoding=="Tight":
-                            jpeg_quality = VNC_JPEG_OPTIONS
+                            jpeg_quality = config.VNC_JPEG_OPTIONS
                         for jpegq in jpeg_quality:
                             cmd = trickle_command(down, up, latency)
-                            cmd += [VNCVIEWER_BIN, "%s::%s" % (IP, PORT),
+                            cmd += [config.VNCVIEWER_BIN, "%s::%s" % (config.IP, config.PORT),
                                    "--ViewOnly",
                                    "--ZlibLevel=%s" % str(zlib),
                                    "--CompressLevel=%s" % str(compression),
@@ -1087,24 +910,39 @@ def test_vnc():
                             tests.append((test_name, "vnc", XVNC_VERSION, VNCVIEWER_VERSION, \
                                           encoding, False, compression, None, False, \
                                           (down,up,latency), x11_test_command, cmd))
-    return with_server(XVNC_SERVER_START_COMMAND, XVNC_SERVER_STOP_COMMANDS, tests, get_vnc_stats)
-
+    return with_server(config.XVNC_SERVER_START_COMMAND, config.XVNC_SERVER_STOP_COMMANDS, tests, get_vnc_stats)
 
 def main():
     #before doing anything, check that the firewall is setup correctly:
     get_iptables_INPUT_count()
     get_iptables_OUTPUT_count()
 
+    #If CUSTOM_PARAMS are supplied on the command line, they override what's in config
+    if (len(sys.argv) > 3):
+        config.CUSTOM_PARAMS = " ".join(sys.argv[3:])
+    config.print_options()
+
     xpra_results = []
-    if TEST_XPRA:
+    if config.TEST_XPRA:
         xpra_results = test_xpra()
     vnc_results = []
-    if TEST_VNC:
+    if config.TEST_VNC:
         vnc_results = test_vnc()
+
+    if (len(sys.argv) > 2):
+        csv_name = sys.argv[2]
+    else:
+        csv_name = None
+
     print("*"*80)
     print("RESULTS:")
     print("")
-    print(", ".join(HEADERS))
+
+    out_lines = []
+    out_line = ", ".join(HEADERS)
+    print out_line
+    out_lines.append(out_line)
+
     def s(x):
         if x is None:
             return ""
@@ -1118,8 +956,16 @@ def main():
             return str(x)
         else:
             return "unhandled-type: %s" % type(x)
+
     for result in xpra_results+vnc_results:
-        print ", ".join([s(x) for x in result])
+        out_line = ", ".join([s(x) for x in result])
+        print(out_line)
+        out_lines.append(out_line)
+
+    if (csv_name != None):
+        with open(csv_name, "w") as csv:
+            for line in out_lines:
+                csv.write(line+"\n")
 
 if __name__ == "__main__":
     main()
