@@ -34,7 +34,7 @@ from xpra.gtk_common.gtk_util import set_tooltip_text, add_close_accel, scaled_i
                                     WIN_POS_CENTER, STATE_NORMAL, \
                                     FILE_CHOOSER_ACTION_SAVE, FILE_CHOOSER_ACTION_OPEN
 from xpra.os_util import thread
-from xpra.client.gtk_base.gtk_tray_menu_base import make_min_auto_menu, make_encodingsmenu, \
+from xpra.client.gtk_base.gtk_tray_menu_base import make_min_auto_menu, make_encodingsmenu, set_use_tray_workaround, \
                                     MIN_QUALITY_OPTIONS, QUALITY_OPTIONS, MIN_SPEED_OPTIONS, SPEED_OPTIONS
 from xpra.client.gtk_base.about import about
 from xpra.client.client_base import SIGNAMES
@@ -390,8 +390,9 @@ class ApplicationWindow:
 
     def encoding_changed(self, *args):
         encoding = self.get_selected_encoding()
-        log("encoding_changed(%s) encoding=%s", args, encoding)
         uses_quality_option = encoding in ["jpeg", "webp", "h264"]
+        log("encoding_changed(%s) uses_quality_option(%s)=%s", args, encoding, uses_quality_option)
+        #to prevent win32 crashes:
         if uses_quality_option:
             self.quality_combo.show()
             self.quality_label.show()
@@ -432,11 +433,16 @@ class ApplicationWindow:
 
 
     def handle_exception(self, e):
+        t = str(e)
+        if self.config.debug:
+            #in debug mode, include the full stacktrace:
+            import traceback
+            t = traceback.format_exc()
         def ui_handle_exception():
             self.reset_client()
             self.set_sensitive(True)
             self.set_info_color(True)
-            self.set_info_text(str(e))
+            self.set_info_text(t)
             self.window.show()
         gobject.idle_add(ui_handle_exception)
 
@@ -484,11 +490,11 @@ class ApplicationWindow:
                 #putty needs those:
                 full_ssh.append("-ssh")
                 full_ssh.append("-agent")
-            if self.config.ssh_port!="22":
+            if str(self.config.ssh_port)!="22":
                 if sys.platform.startswith("win"):
-                    full_ssh += ["-P", self.config.ssh_port]
+                    full_ssh += ["-P", str(self.config.ssh_port)]
                 else:
-                    full_ssh += ["-p", self.config.ssh_port]
+                    full_ssh += ["-p", str(self.config.ssh_port)]
             params["full_ssh"] = full_ssh
             params["password"] = password
             params["display_name"] = "ssh:%s:%s" % (self.config.host, self.config.port)
@@ -531,6 +537,7 @@ class ApplicationWindow:
 
     def do_start_XpraClient(self, conn):
         log("start_XpraClient() client=%s", self.client)
+        self.client.encoding = self.config.encoding
         self.client.setup_connection(conn)
         self.client.init_ui(self.config)
         log("start_XpraClient() client initialized")
@@ -547,6 +554,7 @@ class ApplicationWindow:
         def do_quit(*args):
             self.client.warn_and_quit = warn_and_quit_save
             self.client.quit = quit_save
+            self.client.cleanup()
             self.destroy()
             gtk.main_quit()
         def warn_and_quit_override(exit_code, warning):
@@ -629,6 +637,7 @@ class ApplicationWindow:
         else:
             self.config.mode = "ssh"
         self.config.password = self.password_entry.get_text()
+        log("update_options_from_gui() %s", (self.config.username, self.config.password, self.config.mode, self.config.encryption, self.config.host, self.config.port, self.config.ssh_port, self.config.encoding))
 
     def update_gui_from_config(self):
         #mode:
@@ -640,7 +649,8 @@ class ApplicationWindow:
             self.mode_combo.set_active(2)
         if self.config.encoding:
             index = self.encoding_combo.get_menu().encoding_to_index.get(self.config.encoding, -1)
-            if index>0:
+            log("setting encoding combo to %s / %s", self.config.encoding, index)
+            if index>=0:
                 self.encoding_combo.set_history(index)
         self.username_entry.set_text(self.config.username)
         self.password_entry.set_text(self.config.password)
@@ -727,7 +737,12 @@ def main():
     if debug:
         for x in debug.split(","):
             enable_debug_for(x)
-    app.create_window()
+   #suspend tray workaround for our window widgets:
+    try:
+        set_use_tray_workaround(False)
+        app.create_window()
+    finally:
+        set_use_tray_workaround(True)
     try:
         app.update_gui_from_config()
         if app.config.autoconnect:
