@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -32,8 +32,10 @@ if os.name=="posix" and os.environ.get("XPRA_SET_WORKSPACE", "1")!="0":
     try:
         from xpra.x11.gtk_x11.prop import prop_get, prop_set
         from xpra.x11.bindings.window_bindings import constants, X11WindowBindings  #@UnresolvedImport
+        from xpra.x11.bindings.core_bindings import X11CoreBindings
         from xpra.gtk_common.error import trap
         X11Window = X11WindowBindings()
+        X11Core = X11CoreBindings()
         HAS_X11_BINDINGS = True
 
         SubstructureNotifyMask = constants["SubstructureNotifyMask"]
@@ -57,13 +59,30 @@ except:
     unpremultiply_argb, bgra_to_rgba, byte_buffer_to_buffer  = None, None, None
 
 
+#window types we map to POPUP rather than TOPLEVEL
 #window types we map to POPUP rather than TOPLEVEL
 POPUP_TYPE_HINTS = set((
+                    #"DIALOG",
+                    #"MENU",
+                    #"TOOLBAR",
+                    #"SPLASHSCREEN",
+                    #"UTILITY",
+                    #"DOCK",
+                    #"DESKTOP",
+                    "DROPDOWN_MENU",
+                    "POPUP_MENU",
+                    #"TOOLTIP",
+                    #"NOTIFICATION",
+                    #"COMBO",
+                    #"DND"
+                    ))
+#window types for which we skip window decorations (title bar)
+UNDECORATED_TYPE_HINTS = set((
                     #"DIALOG",
                     "MENU",
                     #"TOOLBAR",
                     "SPLASHSCREEN",
-                    #"UTILITY",
+                    "UTILITY",
                     "DOCK",
                     #"DESKTOP",
                     "DROPDOWN_MENU",
@@ -81,8 +100,10 @@ class GTKKeyEvent(AdHocStruct):
 class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
     def init_window(self, metadata):
+        self._window_state = {}
+        self._maximized = False
+        self._iconfified = False
         self._fullscreen = None
-        self._iconified = False
         self._resize_counter = 0
         self._window_workspace = self._client_properties.get("workspace", -1)
         workspacelog("init_window(..) workspace=%s", self._window_workspace)
@@ -91,7 +112,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         self._can_set_workspace = HAS_X11_BINDINGS and CAN_SET_WORKSPACE
 
     def _is_popup(self, metadata):
-        #decide if the window type is POPUP or NORMAL:
+        #decide if the window type is POPUP or NORMAL
         if self._override_redirect:
             return True
         window_types = metadata.get("window-type", [])
@@ -102,6 +123,28 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             return True
         return False
 
+    def _is_decorated(self, metadata):
+        #decide if the window type is POPUP or NORMAL
+        #(show window decorations or not)
+        if self._override_redirect:
+            return False
+        decorations = metadata.get("decorations")
+        if decorations is not None:
+            #honour the flag given by the server:
+            return bool(decorations)
+        #older servers don't tell us if we need decorations, so take a guess:
+        #skip decorations for any non-normal non-dialog window that is transient for another window:
+        window_types = metadata.get("window-type", [])
+        if ("NORMAL" not in window_types) and ("DIALOG" not in window_types) and metadata.intget("transient-for", -1)>0:
+            return False
+        undecorated_types = list(UNDECORATED_TYPE_HINTS.intersection(window_types))
+        log("undecorated_types(%s)=%s", window_types, undecorated_types)
+        if undecorated_types:
+            log("not decorating window type %s", undecorated_types)
+            return False
+        return True
+
+
     def setup_window(self):
         self.set_app_paintable(True)
         self.add_events(self.WINDOW_EVENT_MASK)
@@ -155,34 +198,53 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
 
     def window_state_updated(self, widget, event):
-        self._fullscreen = bool(event.new_window_state & self.WINDOW_STATE_FULLSCREEN)
-        maximized = bool(event.new_window_state & self.WINDOW_STATE_MAXIMIZED)
-        iconified = bool(event.new_window_state & self.WINDOW_STATE_ICONIFIED)
-        self._client_properties["maximized"] = maximized
-        log("%s.window_state_updated(%s, %s) new_window_state=%s, fullscreen=%s, maximized=%s, iconified=%s", self, widget, repr(event), event.new_window_state, self._fullscreen, maximized, iconified)
-        if iconified!=self._iconified:
-            #handle iconification as map events:
-            assert not self._override_redirect
-            if iconified:
-                assert not self._iconified
-                #usually means it is unmapped
-                self._iconified = True
-                self._unfocus()
-                if not self._override_redirect:
-                    self.send("unmap-window", self._id, True)
-            else:
-                assert not iconified and self._iconified
-                self._iconified = False
-                self.process_map_event()
+        log("%s.window_state_updated(%s, %s) changed_mask=%s, new_window_state=%s", self, widget, repr(event), event.changed_mask, event.new_window_state)
+        if event.changed_mask & self.WINDOW_STATE_FULLSCREEN:
+            fullscreen = bool(event.new_window_state & self.WINDOW_STATE_FULLSCREEN)
+            log("fullscreen=%s (was %s)", fullscreen, self._fullscreen)
+            if fullscreen!=self._fullscreen:
+                self._window_state["fullscreen"] = fullscreen
+                self._fullscreen = fullscreen
+        if event.changed_mask & self.WINDOW_STATE_MAXIMIZED:
+            #this may get sent now as part of map_event code below (and it is irrelevant for the unmap case),
+            #or when we get the configure event - which should come straight after
+            #if we're changing the maximized state
+            maximized = bool(event.new_window_state & self.WINDOW_STATE_MAXIMIZED)
+            log("maximized=%s (was %s)", maximized, self._maximized)
+            if maximized!=self._maximized:
+                self._maximized = maximized
+                self._window_state["maximized"] = maximized
+        if event.changed_mask & self.WINDOW_STATE_ICONIFIED:
+            iconified = bool(event.new_window_state & self.WINDOW_STATE_ICONIFIED)
+            log("iconified=%s (was %s)", iconified, self._iconified)
+            if iconified!=self._iconified:
+                self._iconified = iconified
+                #handle iconification as map events:
+                assert not self._override_redirect
+                if iconified:
+                    #usually means it is unmapped
+                    self._unfocus()
+                    if not self._override_redirect:
+                        #tell server, but wait a bit to try to prevent races:
+                        def tell_server():
+                            if self._iconified:
+                                self.send("unmap-window", self._id, True)
+                                self._window_state = {}
+                        self.timeout_add(150, tell_server)
+                else:
+                    self.process_map_event()
+        if self._window_state:
+            def send_updated_window_state():
+                if self._window_state and self.get_window():
+                    self.process_configure_event()
+            self.timeout_add(25, send_updated_window_state)
 
     def set_fullscreen(self, fullscreen):
-        if self._fullscreen is None or self._fullscreen!=fullscreen:
-            #note: the "_fullscreen" flag is updated by the window-state-event, not here
-            log("%s.set_fullscreen(%s)", self, fullscreen)
-            if fullscreen:
-                self.fullscreen()
-            else:
-                self.unfullscreen()
+        log("%s.set_fullscreen(%s)", self, fullscreen)
+        if fullscreen:
+            self.fullscreen()
+        else:
+            self.unfullscreen()
 
     def set_xid(self, xid):
         if HAS_X11_BINDINGS and self.is_realized():
@@ -288,6 +350,21 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         return -1
 
 
+    def initiate_moveresize(self, x_root, y_root, direction, button, source_indication):
+        log("initiate_moveresize%s", (x_root, y_root, direction, button, source_indication))
+        assert HAS_X11_BINDINGS, "cannot handle initiate-moveresize without X11 bindings"
+        event_mask = SubstructureNotifyMask | SubstructureRedirectMask
+        def wm_moveresize():
+            from xpra.gtk_common.gobject_compat import get_xid
+            root = self.get_window().get_screen().get_root_window()
+            root_xid = get_xid(root)
+            xwin = get_xid(self.get_window())
+            X11Core.UngrabPointer()
+            X11Window.sendClientMessage(root_xid, xwin, False, event_mask, "_NET_WM_MOVERESIZE",
+                  x_root, y_root, direction, button, source_indication)
+        trap.call_synced(wm_moveresize)
+
+
     def apply_transient_for(self, wid):
         if wid==-1:
             #root is a gdk window, so we need to ensure we have one
@@ -349,8 +426,10 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
     def process_map_event(self):
         x, y, w, h = self.get_window_geometry()
+        state = self._window_state
         props = self._client_properties
         self._client_properties = {}
+        self._window_state = {}
         if not self._been_mapped:
             #this is the first time around, so set the workspace:
             workspace = self.set_workspace()
@@ -364,8 +443,8 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             workspacelog("map event: been_mapped=%s, changed workspace from %s to %s", self._been_mapped, self._window_workspace, workspace)
             self._window_workspace = workspace
             props["workspace"] = workspace
-        log("map-window for wid=%s with client props=%s", self._id, props)
-        self.send("map-window", self._id, x, y, w, h, props)
+        log("map-window for wid=%s with client props=%s, state=%s", self._id, props, state)
+        self.send("map-window", self._id, x, y, w, h, props, state)
         self._pos = (x, y)
         self._size = (w, h)
         self.idle_add(self._focus_change, "initial")
@@ -374,7 +453,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
     def do_configure_event(self, event):
         log("%s.do_configure_event(%s)", self, event)
         gtk.Window.do_configure_event(self, event)
-        if not self._override_redirect:
+        if not self._override_redirect and not self._iconified:
             self.process_configure_event()
 
     def process_configure_event(self):
@@ -384,8 +463,10 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         ox, oy = self._pos
         dx, dy = x-ox, y-oy
         self._pos = (x, y)
+        state = self._window_state
         props = self._client_properties
         self._client_properties = {}
+        self._window_state = {}
         if self._been_mapped:
             #if the window has been mapped already, the workspace should be set:
             props["screen"] = self.get_screen().get_number()
@@ -394,9 +475,8 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
                 workspacelog("configure event: changed workspace from %s to %s", self._window_workspace, workspace)
                 self._window_workspace = workspace
                 props["workspace"] = workspace
-        packet = ["configure-window", self._id, x, y, w, h, props]
-        if self._resize_counter>0:
-            packet.append(self._resize_counter)
+        packet = ["configure-window", self._id, x, y, w, h, props, self._resize_counter, state]
+        log("%s", packet)
         self.send(*packet)
         if dx!=0 or dy!=0:
             #window has moved, also move any child OR window:
@@ -456,11 +536,13 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
 
     def do_unmap_event(self, event):
+        log("do_unmap_event(%s)", event)
         self._unfocus()
         if not self._override_redirect:
             self.send("unmap-window", self._id, False)
 
     def do_delete_event(self, event):
+        log("do_delete_event(%s)", event)
         self.send("close-window", self._id)
         return True
 
