@@ -44,7 +44,7 @@ try:
 except:
     ALL_CLIPBOARDS = []
 
-FAKE_BROKEN_CONNECTION = os.environ.get("XPRA_FAKE_BROKEN_CONNECTION", "0")=="1"
+FAKE_BROKEN_CONNECTION = int(os.environ.get("XPRA_FAKE_BROKEN_CONNECTION", "0"))
 PING_TIMEOUT = int(os.environ.get("XPRA_PING_TIMEOUT", "60"))
 UNGRAB_KEY = os.environ.get("XPRA_UNGRAB_KEY", "Escape")
 
@@ -639,7 +639,7 @@ class UIXpraClient(XpraClientBase):
             log.info("sending updated screen size to server: %sx%s with %s screens", root_w, root_h, len(ss))
             log_screen_sizes(root_w, root_h, ss)
             self.send("desktop_size", *screen_settings)
-            self.last_screen_settings = screen_settings
+            self._last_screen_settings = screen_settings
             #update the max packet size (may have gone up):
             self.set_max_packet_size()
         #update via timer so the data is more likely to be final (up to date) when we query it,
@@ -912,6 +912,8 @@ class UIXpraClient(XpraClientBase):
             "auto_refresh_delay"        : int(self.auto_refresh_delay*1000),
             "windows"                   : self.windows_enabled,
             "window.raise"              : True,
+            #only implemented on posix with the gtk client:
+            "window.initiate-moveresize": False,
             "raw_window_icons"          : True,
             "system_tray"               : self.client_supports_system_tray,
             "xsettings-tuple"           : True,
@@ -928,12 +930,13 @@ class UIXpraClient(XpraClientBase):
         for x in compression.ALL_COMPRESSORS:
             capabilities["encoding.rgb_%s" % x] = x in compression.get_enabled_compressors()
 
-        control_commands = ["show_session_info", "enable_bencode", "enable_zlib"]
+        control_commands = ["show_session_info"]
         for x in compression.get_enabled_compressors():
             control_commands.append("enable_"+x)
         for x in packet_encoding.get_enabled_encoders():
             control_commands.append("enable_"+x)
         capabilities["control_commands"] = control_commands
+        log("control_commands=%s", control_commands)
         for k,v in codec_versions.items():
             capabilities["encoding.%s.version" % k] = v
         if self.encoding:
@@ -1046,28 +1049,32 @@ class UIXpraClient(XpraClientBase):
 
     def check_server_echo(self, ping_sent_time):
         last = self._server_ok
-        self._server_ok = not FAKE_BROKEN_CONNECTION and self.last_ping_echoed_time>=ping_sent_time
+        if FAKE_BROKEN_CONNECTION>0:
+            self._server_ok = (int(time.time()) % FAKE_BROKEN_CONNECTION) <= (FAKE_BROKEN_CONNECTION//2)
+        else:
+            self._server_ok = not FAKE_BROKEN_CONNECTION and self.last_ping_echoed_time>=ping_sent_time
         if last!=self._server_ok and not self._server_ok:
             log.info("server is not responding, drawing spinners over the windows")
             def timer_redraw():
                 if self._protocol is None:
                     #no longer connected!
                     return False
+                ok = self.server_ok()
                 self.redraw_spinners()
-                if self.server_ok():
+                if ok:
                     log.info("server is OK again")
-                    return False
-                return True
-            self.redraw_spinners()
+                return not ok
+            self.idle_add(self.redraw_spinners)
             self.timeout_add(100, timer_redraw)
         return False
 
     def redraw_spinners(self):
         #draws spinner on top of the window, or not (plain repaint)
         #depending on whether the server is ok or not
+        ok = self.server_ok()
         for w in self._id_to_window.values():
             if not w.is_tray():
-                w.spinner(self.server_ok())
+                w.spinner(ok)
 
     def check_echo_timeout(self, ping_time):
         log("check_echo_timeout(%s) last_ping_echoed_time=%s", ping_time, self.last_ping_echoed_time)
@@ -1381,14 +1388,14 @@ class UIXpraClient(XpraClientBase):
             args = packet[2:]
             log("calling show_session_info%s on server request", args)
             self.show_session_info(*args)
-        elif command in ("enable_%x" for x in compression.get_enabled_compressors()):
+        elif command in ("enable_%s" % x for x in compression.get_enabled_compressors()):
             compressor = command.split("_")[1]
             log.info("switching to %s on server request", compressor)
             self._protocol.enable_compressor(compressor)
-        elif command in ("enable_%x" for x in packet_encoding.get_enabled_encoders()):
-            packet_encoding = command.split("_")[1]
-            log.info("switching to %s on server request", packet_encoding)
-            self._protocol.enable_bencode()
+        elif command in ("enable_%s" % x for x in packet_encoding.get_enabled_encoders()):
+            pe = command.split("_")[1]
+            log.info("switching to %s on server request", pe)
+            self._protocol.enable_encoder(pe)
         elif command=="name":
             assert len(args)>=3
             self.session_name = args[2]
@@ -1580,6 +1587,11 @@ class UIXpraClient(XpraClientBase):
         codec, data, metadata = packet[1:4]
         codec = bytestostr(codec)
         metadata = typedict(metadata)
+        seq = metadata.intget("sequence", -1)
+        if self.min_sound_sequence>0 and seq>=0 and seq<self.min_sound_sequence:
+            soundlog("ignoring sound data with old sequence number %s", seq)
+            return
+        ss = self.sound_sink
         if not self.speaker_enabled:
             if metadata.boolget("start-of-stream"):
                 #server is asking us to start playing sound
@@ -1596,23 +1608,17 @@ class UIXpraClient(XpraClientBase):
             else:
                 soundlog("speaker is now disabled - dropping packet")
                 return
-        elif metadata.boolget("end-of-stream"):
-            if self.sound_sink:
-                soundlog("server sent end-of-stream, closing sound pipeline")
-                self.stop_receiving_sound(False)
-            return
-        seq = metadata.intget("sequence", -1)
-        if self.min_sound_sequence>0 and seq<self.min_sound_sequence:
-            soundlog("ignoring sound data with old sequence number %s", seq)
-            return
-        ss = self.sound_sink
-        if ss is not None and codec!=self.sound_sink.codec:
-            log.error("sound codec change not supported! (from %s to %s)", ss.codec, codec)
-            ss.stop()
-            return
         if ss is None:
             soundlog("no sound sink to process sound data, dropping it")
             return
+        if metadata.boolget("end-of-stream"):
+            soundlog("server sent end-of-stream, closing sound pipeline")
+            self.stop_receiving_sound(False)
+            return
+        if codec!=ss.codec:
+            log.error("sound codec change not supported! (from %s to %s)", ss.codec, codec)
+            ss.stop()
+            return
         elif ss.get_state()=="stopped":
             soundlog("sound data received, sound sink is stopped - starting it")
             ss.start()
@@ -1701,7 +1707,7 @@ class UIXpraClient(XpraClientBase):
     def make_new_window(self, wid, x, y, w, h, metadata, override_redirect, client_properties):
         metadata = typedict(metadata)
         client_window_classes = self.get_client_window_classes(metadata, override_redirect)
-        group_leader_window = self.get_group_leader(metadata, override_redirect)
+        group_leader_window = self.get_group_leader(wid, metadata, override_redirect)
         #workaround for "popup" OR windows without a transient-for (like: google chrome popups):
         #prevents them from being pushed under other windows on OSX
         #find a "transient-for" value using the pid to find a suitable window
@@ -1733,7 +1739,7 @@ class UIXpraClient(XpraClientBase):
         window.show()
         return window
 
-    def get_group_leader(self, metadata, override_redirect):
+    def get_group_leader(self, wid, metadata, override_redirect):
         #subclasses that wish to implement the feature may override this method
         return None
 
@@ -1888,6 +1894,12 @@ class UIXpraClient(XpraClientBase):
         #only implemented in gtk2 for now
         pass
 
+    def _process_initiate_moveresize(self, packet):
+        wid = packet[1]
+        window = self._id_to_window.get(wid)
+        if window:
+            window.initiate_moveresize(*packet[2:7])
+
     def _process_window_metadata(self, packet):
         wid, metadata = packet[1:3]
         window = self._id_to_window.get(wid)
@@ -1944,8 +1956,10 @@ class UIXpraClient(XpraClientBase):
             pass
         assert maxw>0 and maxh>0 and maxw<32768 and maxh<32768, "problems calculating maximum desktop size: %sx%s" % (maxw, maxh)
         #full screen at 32bits times 4 for safety
-        self._protocol.max_packet_size = maxw*maxh*4*4
-        log("set maximum packet size to %s", self._protocol.max_packet_size)
+        p = self._protocol
+        if p:
+            p.max_packet_size = maxw*maxh*4*4
+            log("set maximum packet size to %s", p.max_packet_size)
 
 
     def init_authenticated_packet_handlers(self):
@@ -1963,6 +1977,7 @@ class UIXpraClient(XpraClientBase):
             "new-override-redirect":self._process_new_override_redirect,
             "new-tray":             self._process_new_tray,
             "raise-window":         self._process_raise_window,
+            "initiate-moveresize":  self._process_initiate_moveresize,
             "window-move-resize":   self._process_window_move_resize,
             "window-resized":       self._process_window_resized,
             "cursor":               self._process_cursor,
