@@ -233,7 +233,9 @@ class WindowBackingBase(object):
             else:
                 img = img.convert("RGB")
 
-        raw_data = img.tostring("raw", img.mode)
+        #use tobytes() if present, fallback to tostring():
+        data_fn = getattr(img, "tobytes", getattr(img, "tostring"))
+        raw_data = data_fn("raw", img.mode)
         paint_options = typedict(options)
         if img.mode=="RGB":
             #PIL flattens the data to a continuous straightforward RGB format:
@@ -282,7 +284,10 @@ class WindowBackingBase(object):
         except KeyboardInterrupt:
             raise
         except:
-            log.error("do_paint_rgb24 error", exc_info=True)
+            if not self._backing:
+                log("paint error on closed backing ignored")
+            else:
+                log.error("do_paint_rgb24 error", exc_info=True)
             fire_paint_callbacks(callbacks, False)
 
     def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
