@@ -14,7 +14,7 @@ if sys.version > '3':
 
 #these codecs may well not load because we
 #do not require the libraries to be installed
-NOWARN = ["nvenc", "opencl"]
+NOWARN = ["opencl"]
 
 codec_errors = {}
 codecs = {}
@@ -105,9 +105,6 @@ def load_codecs():
     codec_import_check("enc_x265", "x265 encoder", "xpra.codecs.enc_x265", "xpra.codecs.enc_x265.encoder", "Encoder")
     add_codec_version("x265", "xpra.codecs.enc_x265.encoder")
 
-    codec_import_check("nvenc", "nvenc encoder", "xpra.codecs.nvenc", "xpra.codecs.nvenc.encoder", "Encoder")
-    add_codec_version("nvenc", "xpra.codecs.nvenc.encoder")
-
     codec_import_check("csc_swscale", "swscale colorspace conversion", "xpra.codecs.csc_swscale", "xpra.codecs.csc_swscale.colorspace_converter", "ColorspaceConverter")
     add_codec_version("swscale", "xpra.codecs.csc_swscale.colorspace_converter")
 
@@ -165,7 +162,7 @@ NEW_ENCODING_NAMES_TO_OLD = {"h264" : "x264", "vp8" : "vpx"}
 ALL_OLD_ENCODING_NAMES_TO_NEW = {"x264" : "h264", "vpx" : "vp8", "rgb24" : "rgb"}
 ALL_NEW_ENCODING_NAMES_TO_OLD = {"h264" : "x264", "vp8" : "vpx", "rgb" : "rgb24"}
 
-ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", "nvenc", \
+ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", \
             "csc_swscale", "csc_cython", "csc_opencl", \
             "dec_avcodec", "dec_avcodec2", \
             "enc_webp", \
@@ -174,6 +171,8 @@ ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", "nvenc", \
 #note: this is just for defining the order of encodings,
 #so we have both core encodings (rgb24/rgb32) and regular encodings (rgb) in here:
 PREFERED_ENCODING_ORDER = ["h264", "vp8", "png", "png/P", "png/L", "webp", "rgb", "rgb24", "rgb32", "jpeg", "h265", "vp9"]
+#encoding order for edges (usually one pixel high or wide):
+EDGE_ENCODING_ORDER = ["rgb24", "rgb32", "jpeg", "png", "webp", "png/P", "png/L", "rgb"]
 
 from xpra.net import compression
 RGB_COMP_OPTIONS  = ["Raw RGB"]
