@@ -18,7 +18,6 @@ from xpra.codecs.loader import get_codec, get_codec_error, load_codecs
 CODEC_TO_MODULE = {"vpx"        : "vpx",
                    "x264"       : "enc_x264",
                    "x265"       : "enc_x265",
-                   "nvenc"      : "nvenc",
                    "swscale"    : "csc_swscale",
                    "cython"     : "csc_cython",
                    "opencl"     : "csc_opencl",
@@ -45,12 +44,12 @@ def try_import_modules(codec_names):
 
 #all the codecs we know about:
 #try to import the module that contains them (cheap check):
-ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265", "nvenc"])
+ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265"])
 ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "cython", "opencl"])
 NO_GFX_CSC_OPTIONS = [x for x in ALL_CSC_MODULE_OPTIONS if x not in ("opencl", )]
 ALL_VIDEO_DECODER_OPTIONS = try_import_modules(["avcodec", "avcodec2", "vpx"])
 
-PREFERRED_ENCODER_ORDER = ["nvenc", "x264", "vpx", "x265"]
+PREFERRED_ENCODER_ORDER = ["x264", "vpx", "x265"]
 PREFERRED_DECODER_ORDER = ["avcodec", "avcodec2", "vpx"]
 log("video_helper: ALL_VIDEO_ENCODER_OPTIONS=%s", ALL_VIDEO_ENCODER_OPTIONS)
 log("video_helper: ALL_CSC_MODULE_OPTIONS=%s", ALL_CSC_MODULE_OPTIONS)
@@ -63,7 +62,7 @@ log("video_helper: ALL_VIDEO_DECODER_OPTIONS=%s", ALL_VIDEO_DECODER_OPTIONS)
 
 def get_encoder_module_name(x):
         if x.find("enc")>=0:
-            return x            #ie: "nvenc" or "enc_vpx"
+            return x            #ie: "enc_vpx"
         return "enc_"+x         #ie: "enc_x264"
 
 def get_decoder_module_name(x):
@@ -130,9 +129,14 @@ class VideoHelper(object):
 
     def set_modules(self, video_encoders=[], csc_modules=[], video_decoders=[]):
         assert not self._initialized, "too late to set modules, the helper is already initialized!"
-        self.video_encoders = [x for x in video_encoders if x in ALL_VIDEO_ENCODER_OPTIONS]
-        self.csc_modules    = [x for x in csc_modules    if x in ALL_CSC_MODULE_OPTIONS]
-        self.video_decoders = [x for x in video_decoders if x in ALL_VIDEO_DECODER_OPTIONS]
+        def filt(name, inlist, all_list):
+            notfound = [x for x in inlist if x not in all_list]
+            if notfound:
+                log.warn("ignoring unknown %s: %s", name, ", ".join(notfound))
+            return [x for x in inlist if x in all_list]
+        self.video_encoders = filt("video encoders" , video_encoders,   ALL_VIDEO_ENCODER_OPTIONS)
+        self.csc_modules    = filt("csc modules"    , csc_modules,      ALL_CSC_MODULE_OPTIONS)
+        self.video_decoders = filt("video decoders" , video_decoders,   ALL_VIDEO_DECODER_OPTIONS)
         log("VideoHelper.set_modules(%s, %s, %s) video encoders=%s, csc=%s, video decoders=%s",
             video_encoders, csc_modules, video_decoders, self.video_encoders, self.csc_modules, self.video_decoders)
 
@@ -189,6 +193,18 @@ class VideoHelper(object):
                 for decoder in decoders:
                     decoder_name, _ = decoder
                     d.setdefault("decoding."+encoding+"_to_"+out_csc, []).append(decoder_name)
+        def modstatus(x, def_list, active_list):
+            #the module is present
+            if x in active_list:
+                return "active"
+            elif x in def_list:
+                return "disabled"
+            else:
+                return "not found"
+        for x in ALL_VIDEO_ENCODER_OPTIONS:
+            d["encoding.video-encoder.%s" % x] = modstatus(x, get_DEFAULT_VIDEO_ENCODERS(), self.video_encoders)
+        for x in ALL_CSC_MODULE_OPTIONS:
+            d["encoding.csc-module.%s" % x] = modstatus(x, get_DEFAULT_CSC_MODULES(), self.csc_modules)
         return d
 
     def init(self):
@@ -242,14 +258,16 @@ class VideoHelper(object):
         encoder_module = get_codec(encoder_name)
         log("init_video_encoder_option(%s) module=%s", encoder_name, encoder_module)
         if not encoder_module:
-            log.warn("video encoder %s could not be loaded: %s", encoder_name, get_codec_error(encoder_name))
+            log.warn("video encoder '%s' could not be loaded:", encoder_name)
+            log.warn(" %s", get_codec_error(encoder_name))
             return
         encoder_type = encoder_module.get_type()
         try:
             encoder_module.init_module()
             self._cleanup_modules.append(encoder_module)
         except Exception, e:
-            log.warn("cannot use %s module %s: %s", encoder_type, encoder_module, e, exc_info=True)
+            log("exception in %s module initialization %s: %s", encoder_type, encoder_module.init_module, e, exc_info=True)
+            log.warn("Warning: %s video encoder failed: %s", encoder_type, e)
             return
         colorspaces = encoder_module.get_input_colorspaces()
         log("init_video_encoder_option(%s) %s input colorspaces=%s", encoder_module, encoder_type, colorspaces)
@@ -291,6 +309,7 @@ class VideoHelper(object):
             csc_module.init_module()
             self._cleanup_modules.append(csc_module)
         except Exception, e:
+            log("exception in %s module initialization %s: %s", csc_type, csc_module.init_module, e, exc_info=True)
             log.warn("cannot use %s module %s: %s", csc_type, csc_module, e)
             return
         in_cscs = csc_module.get_input_colorspaces()
@@ -326,6 +345,7 @@ class VideoHelper(object):
             decoder_module.init_module()
             self._cleanup_modules.append(decoder_module)
         except Exception, e:
+            log("exception in %s module initialization %s: %s", encoder_type, decoder_module.init_module, e, exc_info=True)
             log.warn("cannot use %s module %s: %s", encoder_type, decoder_module, e, exc_info=True)
             return
         encodings = decoder_module.get_encodings()
