@@ -97,6 +97,7 @@ class Protocol(object):
             self._process_packet_cb = process_packet_cb
         self._write_queue = Queue(1)
         self._read_queue = Queue(20)
+        self._read_queue_put = self._read_queue.put
         # Invariant: if .source is None, then _source_has_more == False
         self._get_packet_cb = get_packet_cb
         #counters:
@@ -158,6 +159,7 @@ class Protocol(object):
         exited = True
         for t in (self._read_thread, self._write_thread):
             if t.isAlive():
+                log.warn("%s thread of %s has not yet exited (timeout=%s)", t.name, self._conn, timeout)
                 exited = False
                 break
         return exited
@@ -206,7 +208,10 @@ class Protocol(object):
             info["compressor"] = compression.get_compressor_name(self._compress)
         e = self._encoder
         if e:
-            info["encoder"] = packet_encoding.get_encoder_name(self._encoder)
+            if self._encoder==self.noencode:
+                info["encoder"] = "noencode"
+            else:
+                info["encoder"] = packet_encoding.get_encoder_name(self._encoder)
         for k,v in self.send_aliases.items():
             info["send_alias." + str(k)] = v
             info["send_alias." + str(v)] = k
@@ -375,12 +380,13 @@ class Protocol(object):
             self.enable_compressor("none")
             return
         opts = compression.get_enabled_compressors(order=compression.PERFORMANCE_ORDER)
+        defaults = {"zlib" : True}
         for c in opts:      #ie: [zlib, lz4, lzo]
-            if caps.boolget(c):
+            if caps.boolget(c, defaults.get(c)):
                 self.enable_compressor(c)
                 return
         log.warn("compression disabled: no matching compressor found")
-        self.enable_nocompress()
+        self.enable_compressor("none")
 
     def enable_compressor(self, compressor):
         self._compress = compression.get_compressor(compressor)
@@ -449,7 +455,7 @@ class Protocol(object):
                 #replace this item with an empty string placeholder:
                 packet[i] = ''
             elif ti!=str:
-                log.warn("unexpected data type in %s packet: %s", packet[0], ti)
+                log.warn("unexpected data type %s in %s packet: %s", ti, packet[0], repr_ellipsized(item))
         #now the main packet (or what is left of it):
         packet_type = packet[0]
         self.output_stats[packet_type] = self.output_stats.get(packet_type, 0)+1
@@ -538,7 +544,8 @@ class Protocol(object):
     def _read(self):
         buf = self._conn.read(READ_BUFFER_SIZE)
         #log("read thread: got data of size %s: %s", len(buf), repr_ellipsized(buf))
-        self._read_queue.put(buf)
+        #add to the read queue (or whatever takes its place - see steal_connection)
+        self._read_queue_put(buf)
         if not buf:
             log("read thread: eof")
             self.close()
@@ -863,13 +870,23 @@ class Protocol(object):
         self.terminate_queue_threads()
         self.idle_add(self.clean)
 
-    def steal_connection(self):
+    def steal_connection(self, read_callback=None):
         #so we can re-use this connection somewhere else
         #(frees all protocol threads and resources)
+        #Note: this method can only be used with non-blocking sockets,
+        #and if more than one packet can arrive, the read_callback should be used
+        #to ensure that no packets get lost.
+        #The caller must call wait_for_io_threads_exit() to ensure that this
+        #class is no longer reading from the connection before it can re-use it
         assert not self._closed
+        if read_callback:
+            self._read_queue_put = read_callback
         conn = self._conn
         self._closed = True
         self._conn = None
+        if conn:
+            #this ensures that we exit the untilConcludes() read/write loop
+            conn.set_active(False)
         self.terminate_queue_threads()
         return conn
 
@@ -880,6 +897,7 @@ class Protocol(object):
         self._write_thread = None
         self._read_thread = None
         self._read_parser_thread = None
+        self._write_format_thread = None
         self._process_packet_cb = None
 
     def terminate_queue_threads(self):
