@@ -6,15 +6,11 @@
 
 from xpra.log import Logger
 log = Logger("posix")
+eventlog = Logger("events", "posix")
 
-try:
-    import gtk.gdk
-    from xpra.x11.gtk_x11.error import trap, XError
-    from xpra.x11.bindings import X11KeyboardBindings       #@UnresolvedImport
-    device_bell = X11KeyboardBindings().device_bell
-except:
-    device_bell = None
+from xpra.gtk_common.gobject_compat import get_xid, is_gtk3
 
+device_bell = None
 
 def get_native_notifier_classes():
     ncs = []
@@ -43,24 +39,43 @@ def get_native_system_tray_classes():
     #appindicator can be used for both
     return get_native_tray_classes()
 
+
 def system_bell(window, device, percent, pitch, duration, bell_class, bell_id, bell_name):
     global device_bell
-    if device_bell is None:
+    if device_bell is False:
+        #failed already
+        return False
+    def x11_bell():
+        global device_bell
+        if device_bell is None:
+            #try to load it:
+            from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings       #@UnresolvedImport
+            device_bell = X11KeyboardBindings().device_bell
+        device_bell(get_xid(window), device, bell_class, bell_id, percent, bell_name)
+    try:
+        from xpra.gtk_common.error import trap, XError
+    except Exception, e:
+        log.error("error using device_bell: %s, switching native X11 bell support off", e)
+        device_bell = False
         return False
     try:
-        trap.call_synced(device_bell, window.xid, device, bell_class, bell_id, percent, bell_name)
+        trap.call_synced(x11_bell)
         return  True
     except XError, e:
         log.error("error using device_bell: %s, switching native X11 bell support off", e)
-        device_bell = None
+        device_bell = False
         return False
 
 
 class ClientExtras(object):
-    def __init__(self, client):
+    def __init__(self, client, opts):
         self.client = client
         self._xsettings_watcher = None
         self._root_props_watcher = None
+        self.system_bus = None
+        self.upower_resuming_match = None
+        self.upower_sleeping_match = None
+        self.login1_match = None
         if client.xsettings_enabled:
             self.setup_xprops()
         self.setup_dbus_signals()
@@ -73,12 +88,23 @@ class ClientExtras(object):
         if self._root_props_watcher:
             self._root_props_watcher.cleanup()
             self._root_props_watcher = None
-
+        if self.system_bus:
+            bus = self.system_bus
+            log("cleanup() system bus=%s, matches: %s", bus, (self.upower_resuming_match, self.upower_sleeping_match, self.login1_match))
+            self.system_bus = None
+            if self.upower_resuming_match:
+                bus._clean_up_signal_match(self.upower_resuming_match)
+            if self.upower_sleeping_match:
+                bus._clean_up_signal_match(self.upower_sleeping_match)
+            if self.login1_match:
+                bus._clean_up_signal_match(self.login1_match)
 
     def resuming_callback(self, *args):
+        eventlog("resuming_callback%s", args)
         self.client.resume()
 
     def sleeping_callback(self, *args):
+        eventlog("sleeping_callback%s", args)
         self.client.suspend()
 
 
@@ -86,40 +112,47 @@ class ClientExtras(object):
         try:
             from xpra.x11.dbus_common import init_system_bus
             bus = init_system_bus()
+            self.system_bus = bus
+            log("setup_dbus_signals() system bus=%s", bus)
         except Exception, e:
             log.warn("dbus setup error: %s", e)
             return
 
         #the UPower signals:
         try:
-            iface_name  = 'org.freedesktop.UPower'
             bus_name    = 'org.freedesktop.UPower'
-            bus.add_signal_receiver(self.resuming_callback, 'Resuming', iface_name, bus_name)
-            bus.add_signal_receiver(self.sleeping_callback, 'Sleeping', iface_name, bus_name)
-            log("listening for 'Resuming' and 'Sleeping' signals on %s", iface_name)
+            log("bus has owner(%s)=%s", bus_name, bus.name_has_owner(bus_name))
+            iface_name  = 'org.freedesktop.UPower'
+            self.upower_resuming_match = bus.add_signal_receiver(self.resuming_callback, 'Resuming', iface_name, bus_name)
+            self.upower_sleeping_match = bus.add_signal_receiver(self.sleeping_callback, 'Sleeping', iface_name, bus_name)
+            eventlog("listening for 'Resuming' and 'Sleeping' signals on %s", iface_name)
         except Exception, e:
-            log("failed to setup UPower event listener: %s", e)
+            eventlog("failed to setup UPower event listener: %s", e)
 
         #the "logind" signals:
         try:
+            bus_name    = 'org.freedesktop.login1'
+            log("bus has owner(%s)=%s", bus_name, bus.name_has_owner(bus_name))
             def sleep_event_handler(suspend):
                 if suspend:
                     self.sleeping_callback()
                 else:
                     self.resuming_callback()
             iface_name  = 'org.freedesktop.login1.Manager'
-            bus_name    = 'org.freedesktop.login1'
-            bus.add_signal_receiver(sleep_event_handler, 'PrepareForSleep', iface_name, bus_name)
-            log("listening for 'PrepareForSleep' signal on %s", iface_name)
+            self.login1_match = bus.add_signal_receiver(sleep_event_handler, 'PrepareForSleep', iface_name, bus_name)
+            eventlog("listening for 'PrepareForSleep' signal on %s", iface_name)
         except Exception, e:
-            log("failed to setup login1 event listener: %s", e)
+            eventlog("failed to setup login1 event listener: %s", e)
 
     def setup_xprops(self):
         #wait for handshake to complete:
         self.client.connect("handshake-complete", self.do_setup_xprops)
 
     def do_setup_xprops(self, *args):
-        log.debug("do_setup_xprops(%s)", args)
+        log("do_setup_xprops(%s)", args)
+        if is_gtk3():
+            log("x11 root properties and XSETTINGS are not supported yet with GTK3")
+            return
         ROOT_PROPS = ["RESOURCE_MANAGER", "_NET_WORKAREA", "_NET_CURRENT_DESKTOP"]
         try:
             from xpra.x11.xsettings import XSettingsWatcher
@@ -152,6 +185,7 @@ class ClientExtras(object):
             if not self.client.xsettings_tuple:
                 log.warn("xsettings tuple format not supported, update ignored")
                 return
+            import gtk.gdk
             root = gtk.gdk.get_default_root_window()
             from xpra.x11.gtk_x11.prop import prop_get
             value = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
