@@ -45,7 +45,12 @@ class Keyboard(KeyboardBase):
                     for modifier,keys in mod_mappings.items():
                         for _,keyname in keys:
                             meanings[keyname] = modifier
-                    return  meanings, [], []
+                    #probably a GTK bug? but easier to put here
+                    mod_missing = []
+                    numlock_mod = meanings.get("Num_Lock", [])
+                    if numlock_mod:
+                        mod_missing.append(numlock_mod)
+                    return  meanings, [], mod_missing
             except Exception, e:
                 log.error("failed to use native get_modifier_mappings: %s", e, exc_info=True)
         return self.modifiers_fallback()
