@@ -20,6 +20,7 @@ def debug(*args):
     pass
 
 DEFAULT_XPRA_CONF_FILENAME = os.environ.get("XPRA_CONF_FILENAME", 'xpra.conf')
+DEFAULT_NET_WM_NAME = os.environ.get("XPRA_NET_WM_NAME", "Xpra")
 
 
 ENCRYPTION_CIPHERS = []
@@ -244,6 +245,7 @@ OPTION_TYPES = {
                     "title"             : str,
                     "username"          : str,
                     "auth"              : str,
+                    "wm-name"           : str,
                     "remote-xpra"       : str,
                     "session-name"      : str,
                     "client-toolkit"    : str,
@@ -338,6 +340,7 @@ def get_defaults():
                     "title"             : "@title@ on @client-machine@",
                     "username"          : username,
                     "auth"              : "",
+                    "wm-name"           : DEFAULT_NET_WM_NAME,
                     "remote-xpra"       : "~/.xpra/run-xpra",
                     "session-name"      : "",
                     "client-toolkit"    : "",
@@ -394,7 +397,7 @@ def get_defaults():
                     "exit-with-client"  : False,
                     "exit-ssh"          : True,
                     "opengl"            : OPENGL_DEFAULT,
-                    "mdns"              : os.name=="posix" and not sys.platform.startswith("darwin"),
+                    "mdns"              : False,
                     "swap-keys"         : sys.platform.startswith("darwin"),    #only used on osx
                     "encodings"         : ["all"],
                     "video-encoders"    : ["all"],
