@@ -267,7 +267,8 @@ def do_parse_cmdline(cmdline, defaults):
     parser = ModifiedOptionParser(version="xpra v%s" % XPRA_VERSION,
                           usage="\n" + "".join(command_options))
     hidden_options = {"display" : defaults.display,
-                      "displayfd" : defaults.displayfd}
+                      "displayfd" : defaults.displayfd,
+                      "wm_name" : defaults.wm_name}
     group = optparse.OptionGroup(parser, "Server Options",
                 "These options are only relevant on the server when using the %s mode." %
                 " or ".join(["'%s'" % x for x in server_modes]))
@@ -1106,9 +1107,13 @@ def run_client(error_cb, opts, extra_args, mode):
         from xpra.client.gobject_client_base import DetachXpraClient
         app = DetachXpraClient(connect(), opts)
     else:
-        from xpra.platform.gui import init as gui_init
-        gui_init()
-        app = make_client(error_cb, opts)
+        try:
+            from xpra.platform.gui import init as gui_init
+            gui_init()
+            app = make_client(error_cb, opts)
+        except RuntimeError, e:
+            #exceptions at this point are still initialization exceptions
+            raise InitException(e.message)
         layouts = app.get_supported_window_layouts() or ["default"]
         layouts_str = ", ".join(layouts)
         if opts.window_layout and opts.window_layout.lower()=="help":
@@ -1132,8 +1137,12 @@ def run_client(error_cb, opts, extra_args, mode):
         if hasattr(app, "connect"):
             app.connect("handshake-complete", handshake_complete)
         app.init_ui(opts, extra_args)
-        conn = connect()
-        app.setup_connection(conn)
+        try:
+            conn = connect()
+            app.setup_connection(conn)
+        except Exception, e:
+            app.cleanup()
+            raise e
     return do_run_client(app)
 
 def make_client(error_cb, opts):
@@ -1361,6 +1370,7 @@ def run_stopexit(mode, error_cb, opts, extra_args):
 
     display_desc = pick_display(error_cb, opts, extra_args)
     conn = connect_or_fail(display_desc)
+    app = None
     e = 1
     try:
         if mode=="stop":
@@ -1373,7 +1383,8 @@ def run_stopexit(mode, error_cb, opts, extra_args):
             raise Exception("invalid mode: %s" % mode)
         e = app.run()
     finally:
-        app.cleanup()
+        if app:
+            app.cleanup()
     if display_desc["local"]:
         show_final_state(display_desc["display"])
     else:
