@@ -28,6 +28,11 @@ o0177 = 127
 o0666 = 438
 o0700 = 448
 
+# use process polling with python versions older than 2.7 and 3.0, (because SIGCHLD support is broken)
+# or when the user requests it with the env var:
+USE_PROCESS_POLLING = os.environ.get("XPRA_USE_PROCESS_POLLING")=="1" or sys.version_info<(2, 7) or sys.version_info[:2]==(3, 0)
+
+
 _cleanups = []
 def run_cleanups():
     global _cleanups
@@ -72,21 +77,11 @@ class ChildReaper(object):
         self._ignored_pids = set()
         from xpra.log import Logger
         self._logger = Logger("server", "util")
-        old_python = sys.version_info < (2, 7) or sys.version_info[:2] == (3, 0)
-        if old_python:
+        if USE_PROCESS_POLLING:
             POLL_DELAY = int(os.environ.get("XPRA_POLL_DELAY", 2))
             self._logger.warn("Warning: outdated/buggy version of Python: %s", ".".join(str(x) for x in sys.version_info))
             self._logger.warn("switching to process polling every %s seconds to support 'exit-with-children'", POLL_DELAY)
-            #keep track of process objects:
-            self.processes = []
-            def check_procs():
-                for proc in self.processes:
-                    if proc.poll() is not None:
-                        self.add_dead_pid(proc.pid)
-                        self.check()
-                self.processes = [proc for proc in self.processes if proc.poll() is None]
-                return True
-            gobject.timeout_add(POLL_DELAY*1000, check_procs)
+            gobject.timeout_add(POLL_DELAY*1000, self.check)
         else:
             #with a less buggy python, we can just check the list of pids
             #whenever we get a SIGCHLD
@@ -123,10 +118,12 @@ class ChildReaper(object):
             self._logger("check() pids=%s, dead_pids=%s", pids, self._dead_pids)
             if pids.issubset(self._dead_pids):
                 self._quit()
+                return False
+        return True
 
     def sigchld(self, signum, frame):
-        self._logger("sigchld(%s, %s)", signum, frame)
-        self.reap()
+        gobject.idle_add(self._logger, "sigchld(%s, %s)", signum, frame)
+        gobject.idle_add(self.reap)
 
     def add_dead_pid(self, pid):
         self._logger("add_dead_pid(%s)", pid)
@@ -733,7 +730,11 @@ def start_children(child_reaper, commands, fake_xinerama):
     #disable ubuntu's global menu using env vars:
     env.update({
         "UBUNTU_MENUPROXY"          : "",
-        "QT_X11_NO_NATIVE_MENUBAR"  : "1"})
+        "QT_X11_NO_NATIVE_MENUBAR"  : "1",
+        "MWNOCAPTURE"               : "true",
+        "MWNO_RIT"                  : "true",
+        "MWWM"                      : "allwm",
+    })
     for child_cmd in commands:
         if not child_cmd:
             continue
@@ -795,6 +796,8 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
             if not shadowing and not proxying:
                 display_name_check(display_name)
         else:
+            if proxying:
+                error_cb("you must specify a free virtual display name to use with the proxy server")
             if not opts.displayfd:
                 error_cb("displayfd support is not enabled on this system, you must specify the display to use")
             if opts.use_display:
@@ -888,7 +891,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
         if log_filename0 != log_filename1:
             # we now have the correct log filename, so use it:
             os.rename(log_filename0, log_filename1)
-            stderr.write("Actual log file name is now: %s" % log_filename1)
+            stderr.write("Actual log file name is now: %s\n" % log_filename1)
         stdout.close()
         stderr.close()
 
@@ -957,7 +960,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
 
         #check for an existing window manager:
         from xpra.x11.gtk_x11.wm import wm_check
-        if not wm_check(display, upgrading):
+        if not wm_check(display, opts.wm_name, upgrading):
             return 1
         try:
             # This import is delayed because the module depends on gtk:
