@@ -17,7 +17,7 @@ except Exception, e:
     log("cannot load argb module: %s", e)
     bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba = (None,)*4
 from xpra.codecs.codec_constants import get_PIL_encodings
-from xpra.os_util import StringIOClass
+from xpra.os_util import StringIOClass, memoryview_to_bytes
 from xpra.codecs.loader import get_codec, get_codec_version
 from xpra.os_util import builtins
 _memoryview = builtins.__dict__.get("memoryview")
@@ -77,7 +77,7 @@ def webp_encode(coding, image, rgb_formats, supports_transparency, quality, spee
     encs = get_PIL_encodings(PIL)
     for x in ("webp", "png"):
         if x in encs:
-            return PIL_encode(coding, image, quality, speed, supports_transparency)
+            return PIL_encode(x, image, quality, speed, supports_transparency)
     raise Exception("BUG: cannot use 'webp' encoding and none of the PIL fallbacks are available!")
 
 
@@ -118,7 +118,7 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
             # and convert BGRX to RGB for example (assuming RGB is also supported by the client)
             rows = []
             for y in range(height):
-                rows.append(pixels[stride*y:stride*y+rstride])
+                rows.append(memoryview_to_bytes(pixels[stride*y:stride*y+rstride]))
             pixels = "".join(rows)
             log("rgb_encode: %s pixels re-stride saving %i%% from %s (%s bytes) to %s (%s bytes)", pixel_format, 100-100*el/al, stride, al, rstride, el)
             stride = rstride
@@ -129,7 +129,7 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
     wire_data = raw_data
     level = 0
     algo = "not"
-    if len(pixels)>=256 and (rgb_zlib and compression.use_zlib) or (rgb_lz4 and compression.lz4) or (rgb_lzo and compression.use_lzo):
+    if len(pixels)>=256 and (rgb_zlib and compression.use_zlib) or (rgb_lz4 and compression.use_lz4) or (rgb_lzo and compression.use_lzo):
         level = max(0, min(5, int(115-speed)/20))
         if len(pixels)<1024:
             #fewer pixels, make it more likely we won't bother compressing:
@@ -349,7 +349,9 @@ def rgb_reformat(image, rgb_formats, supports_transparency):
         pixels = pixels.tobytes()
     img = PIL.Image.frombuffer(target_format, (w, h), pixels, "raw", input_format, image.get_rowstride())
     rowstride = w*len(target_format)    #number of characters is number of bytes per pixel!
-    data = img.tostring("raw", target_format)
+    #use tobytes() if present, fallback to tostring():
+    data_fn = getattr(img, "tobytes", getattr(img, "tostring"))
+    data = data_fn("raw", target_format)
     assert len(data)==rowstride*h, "expected %s bytes in %s format but got %s" % (rowstride*h, len(data))
     image.set_pixels(data)
     image.set_rowstride(rowstride)
