@@ -586,8 +586,11 @@ class ServerBase(ServerCore):
             key_repeat = (0, 0)
         else:
             root_w, root_h = self.set_best_screen_size()
-            self.calculate_workarea()
-            self.set_desktop_geometry(dw or root_w, dh or root_h)
+            #prefer desktop size, fallback to screen size:
+            w = dw or root_w
+            h = dh or root_h
+            self.calculate_workarea(w, h)
+            self.set_desktop_geometry(w, h)
             #take the clipboard if no-one else has yet:
             if ss.clipboard_enabled and self._clipboard_helper is not None and \
                 (self._clipboard_client is None or self._clipboard_client.is_closed()):
@@ -676,7 +679,8 @@ class ServerBase(ServerCore):
                 "encoding.generic", "encoding.strict_control",
                 "sound.server_driven",
                 "command_request",
-                "event_request", "server-events")
+                "event_request", "server-events",
+                "sound.eos-sequence")
 
     def make_hello(self, source):
         capabilities = ServerCore.make_hello(self, source)
@@ -850,7 +854,7 @@ class ServerBase(ServerCore):
                 return arg_err("more than 1")
             msg = []
             for csource in sources:
-                msg.append("%s : %s" % (csource, csource.sound_control(*args[1:])))
+                msg.append("%s : %s" % (csource, csource.sound_control(*args)))
             return 0, ", ".join(msg)
         elif command=="suspend":
             for csource in sources:
@@ -1002,11 +1006,11 @@ class ServerBase(ServerCore):
 
     def _process_info_request(self, proto, packet):
         ss = self._server_sources.get(proto)
-        assert ss, "cannot find server source for %s" % proto
-        def info_callback(_proto, info):
-            assert proto==_proto
-            ss.send_info_response(info)
-        self.get_all_info(info_callback, proto, *packet[1:])
+        if ss:
+            def info_callback(_proto, info):
+                assert proto==_proto
+                ss.send_info_response(info)
+            self.get_all_info(info_callback, proto, *packet[1:])
 
     def send_hello_info(self, proto):
         log.info("processing info request from %s", proto._conn)
@@ -1215,10 +1219,12 @@ class ServerBase(ServerCore):
             ss.new_window(ptype, wid, window, x, y, w, h, wprops)
 
 
-    def _screen_size_changed(self, *args):
-        log("_screen_size_changed(%s)", args)
+    def _screen_size_changed(self, screen):
+        log("_screen_size_changed(%s)", screen)
         #randr has resized the screen, tell the client (if it supports it)
-        self.calculate_workarea()
+        w, h = screen.get_width(), screen.get_height()
+        log("new screen dimensions: %s", (w, h))
+        self.calculate_workarea(w, h)
         self.idle_add(self.send_updated_screen_size)
 
     def get_root_window_size(self):
@@ -1267,9 +1273,9 @@ class ServerBase(ServerCore):
             log.info("received updated display dimensions")
             log.info("client root window size is %sx%s with %s displays:", width, height, len(ss.screen_sizes))
             log_screen_sizes(width, height, ss.screen_sizes)
-            self.calculate_workarea()
+            self.calculate_workarea(width, height)
 
-    def calculate_workarea(self):
+    def calculate_workarea(self, w, h):
         raise NotImplementedError()
 
     def set_workarea(self, workarea):
@@ -1441,6 +1447,9 @@ class ServerBase(ServerCore):
             #filter out encoding properties, which are expected to be set everytime:
             ncp = {}
             for k,v in new_client_properties.items():
+                if v is None:
+                    log.warn("removing invalid None property for %s", k)
+                    continue
                 if not k.startswith("encoding"):
                     ncp[k] = v
             log("set_client_properties updating window %s with %s", wid, ncp)
