@@ -17,21 +17,23 @@ scalinglog = Logger("scaling")
 
 
 AUTO_REFRESH_ENCODING = os.environ.get("XPRA_AUTO_REFRESH_ENCODING", "")
-AUTO_REFRESH_THRESHOLD = int(os.environ.get("XPRA_AUTO_REFRESH_THRESHOLD", 95))
+AUTO_REFRESH_THRESHOLD = int(os.environ.get("XPRA_AUTO_REFRESH_THRESHOLD", 100))
 AUTO_REFRESH_QUALITY = int(os.environ.get("XPRA_AUTO_REFRESH_QUALITY", 100))
 AUTO_REFRESH_SPEED = int(os.environ.get("XPRA_AUTO_REFRESH_SPEED", 50))
 
-MAX_PIXELS_PREFER_RGB = 4096
+MAX_PIXELS_PREFER_RGB = int(os.environ.get("XPRA_MAX_PIXELS_PREFER_RGB", 4096))
 
 DELTA = os.environ.get("XPRA_DELTA", "1")=="1"
 MAX_DELTA_SIZE = int(os.environ.get("XPRA_MAX_DELTA_SIZE", "10000"))
 HAS_ALPHA = os.environ.get("XPRA_ALPHA", "1")=="1"
 FORCE_BATCH = os.environ.get("XPRA_FORCE_BATCH", "0")=="1"
 STRICT_MODE = os.environ.get("XPRA_ENCODING_STRICT_MODE", "0")=="1"
+MERGE_REGIONS = os.environ.get("XPRA_MERGE_REGIONS", "1")=="1"
 
 
 from xpra.deque import maxdeque
 from xpra.util import updict
+from xpra.os_util import memoryview_to_bytes
 from xpra.server.window_stats import WindowPerformanceStatistics
 from xpra.simple_stats import add_list_stats
 from xpra.server.batch_delay_calculator import calculate_batch_delay, get_target_speed, get_target_quality
@@ -103,7 +105,7 @@ class WindowSource(object):
         self.generic_encodings = encoding_options.boolget("generic")
         self.supports_transparency = HAS_ALPHA and encoding_options.boolget("transparency")
         self.full_frames_only = encoding_options.boolget("full_frames_only")
-        ropts = set(("png", "webp", "rgb", "jpeg"))     #default encodings for auto-refresh
+        ropts = set(("png", "webp", "rgb24", "rgb32", "jpeg"))     #default encodings for auto-refresh
         if self.webp_leaks:
             ropts.remove("webp")                        #don't use webp if the client is going to leak with it!
         ropts = ropts.intersection(set(self.server_core_encodings)) #ensure the server has support for it
@@ -337,7 +339,8 @@ class WindowSource(object):
         self.cancel_damage()
         self.statistics.reset()
         self.suspended = False
-        self.refresh(window, {"quality" : 100})
+        if not window.is_OR() and not window.is_tray():
+            self.refresh(window, {"quality" : 100})
 
     def refresh(self, window, options={}):
         w, h = window.get_dimensions()
@@ -800,19 +803,25 @@ class WindowSource(object):
         return False
 
     def delayed_region_timeout(self, delayed_region_time):
-        if self._damage_delayed is None:
+        self.timeout_timer = None
+        delayed = self._damage_delayed
+        if delayed is None:
             #delayed region got sent
             return False
-        region_time = self._damage_delayed[0]
+        region_time = delayed[0]
         if region_time!=delayed_region_time:
             #this is a different region
             return False
         #ouch: same region!
-        window      = self._damage_delayed[1]
-        options     = self._damage_delayed[4]
+        window      = delayed[1]
+        options     = delayed[4]
         elapsed = int(1000.0 * (time.time() - region_time))
         log.warn("delayed_region_timeout: region is %ims old, bad connection?", elapsed)
-        #re-try:
+        #re-try: cancel anything pending and do a full quality refresh
+        self.cancel_damage()
+        self.cancel_expire_timer()
+        self.cancel_refresh_timer()
+        self.cancel_soft_timer()
         self._damage_delayed = None
         self.full_quality_refresh(window, options)
         return False
@@ -821,19 +830,19 @@ class WindowSource(object):
         """ send the delayed region for processing if there is no client backlog """
         if not self._damage_delayed:
             log("window %s delayed region already sent", self.wid)
-            return False
+            return
         damage_time = self._damage_delayed[0]
         packets_backlog = self.statistics.get_packets_backlog()
         now = time.time()
         actual_delay = int(1000.0 * (now-damage_time))
         if packets_backlog>0:
-            if actual_delay<self.batch_config.max_delay:
-                log("send_delayed for wid %s, delaying again because of backlog: %s packets, batch delay is %s, elapsed time is %.1f ms",
-                        self.wid, packets_backlog, self.batch_config.delay, actual_delay)
-                #this method will get fired again damage_packet_acked
-                return False
-            else:
-                log.warn("send_delayed for wid %s, elapsed time %.1f is above limit of %.1f - sending now", self.wid, actual_delay, self.batch_config.max_delay)
+            if actual_delay>self.batch_config.timeout_delay:
+                log.warn("send_delayed for wid %s, elapsed time %.1f is above limit of %.1f", self.wid, actual_delay, self.batch_config.timeout_delay)
+                return
+            log("send_delayed for wid %s, delaying again because of backlog: %s packets, batch delay is %s, elapsed time is %.1f ms",
+                    self.wid, packets_backlog, self.batch_config.delay, actual_delay)
+            #this method will fire again from damage_packet_acked
+            return
         else:
             #if we're here, there is no packet backlog, and therefore
             #may_send_delayed() may not be called again by an ACK packet,
@@ -909,46 +918,47 @@ class WindowSource(object):
                 return
 
         regions = list(set(regions))
-        bytes_threshold = ww*wh*self.max_bytes_percent/100
-        pixel_count = sum(rect.width*rect.height for rect in regions)
-        bytes_cost = pixel_count+self.small_packet_cost*len(regions)
-        log("send_delayed_regions: bytes_cost=%s, bytes_threshold=%s, pixel_count=%s", bytes_cost, bytes_threshold, pixel_count)
-        if bytes_cost>=bytes_threshold:
-            #too many bytes to send lots of small regions..
-            if exclude_region is None:
-                send_full_window_update()
-                return
-            #make regions out of the rest of the window area:
-            non_exclude = rectangle(0, 0, ww, wh).substract_rect(exclude_region)
-            #and keep those that have damage areas in them:
-            regions = [x for x in non_exclude if len([y for y in regions if x.intersects_rect(y)])>0]
-            #TODO: should verify that is still better than what we had before..
-
-        elif len(regions)>1:
-            #try to merge all the regions to see if we save anything:
-            merged = regions[0].clone()
-            for r in regions[1:]:
-                merged.merge_rect(r)
-            #remove the exclude region if needed:
-            if exclude_region:
-                merged_rects = merged.substract_rect(exclude_region)
-            else:
-                merged_rects = [merged]
-            merged_pixel_count = sum([r.width*r.height for r in merged_rects])
-            merged_bytes_cost = pixel_count+self.small_packet_cost*len(merged_rects)
-            if merged_bytes_cost<bytes_cost or merged_pixel_count<pixel_count:
-                #better, so replace with merged regions:
-                regions = merged_rects
-
-        #check to see if the total amount of pixels makes us use a fullscreen update instead:
-        if len(regions)>1:
+        if MERGE_REGIONS:
+            bytes_threshold = ww*wh*self.max_bytes_percent/100
             pixel_count = sum(rect.width*rect.height for rect in regions)
-            log("send_delayed_regions: %s regions with %s pixels (coding=%s)", len(regions), pixel_count, coding)
-            actual_encoding = get_encoding(pixel_count)
-            if self.must_encode_full_frame(window, actual_encoding):
-                #use full screen dimensions:
-                self.process_damage_region(damage_time, window, 0, 0, ww, wh, actual_encoding, options)
-                return
+            bytes_cost = pixel_count+self.small_packet_cost*len(regions)
+            log("send_delayed_regions: bytes_cost=%s, bytes_threshold=%s, pixel_count=%s", bytes_cost, bytes_threshold, pixel_count)
+            if bytes_cost>=bytes_threshold:
+                #too many bytes to send lots of small regions..
+                if exclude_region is None:
+                    send_full_window_update()
+                    return
+                #make regions out of the rest of the window area:
+                non_exclude = rectangle(0, 0, ww, wh).substract_rect(exclude_region)
+                #and keep those that have damage areas in them:
+                regions = [x for x in non_exclude if len([y for y in regions if x.intersects_rect(y)])>0]
+                #TODO: should verify that is still better than what we had before..
+    
+            elif len(regions)>1:
+                #try to merge all the regions to see if we save anything:
+                merged = regions[0].clone()
+                for r in regions[1:]:
+                    merged.merge_rect(r)
+                #remove the exclude region if needed:
+                if exclude_region:
+                    merged_rects = merged.substract_rect(exclude_region)
+                else:
+                    merged_rects = [merged]
+                merged_pixel_count = sum(r.width*r.height for r in merged_rects)
+                merged_bytes_cost = merged_pixel_count+self.small_packet_cost*len(merged_rects)
+                if merged_bytes_cost<bytes_cost or merged_pixel_count<pixel_count:
+                    #better, so replace with merged regions:
+                    regions = merged_rects
+    
+            #check to see if the total amount of pixels makes us use a fullscreen update instead:
+            if len(regions)>1:
+                pixel_count = sum(rect.width*rect.height for rect in regions)
+                log("send_delayed_regions: %s regions with %s pixels (coding=%s)", len(regions), pixel_count, coding)
+                actual_encoding = get_encoding(pixel_count)
+                if pixel_count>=ww*wh or self.must_encode_full_frame(window, actual_encoding):
+                    #use full screen dimensions:
+                    self.process_damage_region(damage_time, window, 0, 0, ww, wh, actual_encoding, options)
+                    return
 
         #we're processing a number of regions separately:
         for region in regions:
@@ -1046,7 +1056,8 @@ class WindowSource(object):
         actual_quality = client_options.get("quality", 0)
         if encoding.startswith("png") or encoding.startswith("rgb"):
             actual_quality = 100
-        lossy_csc = client_options.get("csc") in LOSSY_PIXEL_FORMATS
+        #jpeg uses colour subsampling by default, otherwise check the csc format value:
+        lossy_csc = encoding=="jpeg" or client_options.get("csc") in LOSSY_PIXEL_FORMATS
         scaled = client_options.get("scaled_size") is not None
         region = rectangle(x, y, w, h)
         if actual_quality>=AUTO_REFRESH_THRESHOLD and not lossy_csc and not scaled:
@@ -1161,11 +1172,13 @@ class WindowSource(object):
             #choose an encoding:
             ww, wh = window.get_dimensions()
             encoding = self.auto_refresh_encodings[0]
-            encodings = self.get_best_encoding(ww*wh, ww, wh, AUTO_REFRESH_SPEED, AUTO_REFRESH_QUALITY, encoding)
-            refresh_encodings = [x for x in self.auto_refresh_encodings if x in encodings]
-            if refresh_encodings:
-                encoding = refresh_encodings[0]
+            best_encoding = self.get_best_encoding(ww*wh, ww, wh, AUTO_REFRESH_SPEED, AUTO_REFRESH_QUALITY, encoding)
+            refresh_encodings = self.auto_refresh_encodings
+            if best_encoding in refresh_encodings:
+                encoding = best_encoding
             options = self.get_refresh_options()
+            refreshlog("timer_full_refresh() size=%s, encoding=%s, best=%s, auto_refresh_encodings=%s, refresh_encodings=%s, options=%s",
+                            (ww, wh), encoding, best_encoding, self.auto_refresh_encodings, refresh_encodings, options)
             WindowSource.do_send_delayed_regions(self, now, window, regions, encoding, options, exclude_region=self.get_refresh_exclude())
         return False
 
@@ -1243,12 +1256,14 @@ class WindowSource(object):
         log("packet decoding sequence %s for window %s %sx%s took %.1fms", damage_packet_sequence, self.wid, width, height, decode_time/1000.0)
         if decode_time>0:
             self.statistics.client_decode_time.append((time.time(), width*height, decode_time))
+        elif decode_time<0:
+            self.client_decode_error(decode_time)
         pending = self.statistics.damage_ack_pending.get(damage_packet_sequence)
         if pending is None:
             log("cannot find sent time for sequence %s", damage_packet_sequence)
             return
         del self.statistics.damage_ack_pending[damage_packet_sequence]
-        if decode_time:
+        if decode_time>0:
             start_send_at, start_bytes, end_send_at, end_bytes, pixels = pending
             bytecount = end_bytes-start_bytes
             #it is possible, though very unlikely,
@@ -1256,14 +1271,16 @@ class WindowSource(object):
             #damage_packet_sent, so we must validate the data:
             if bytecount>0 and end_send_at>0:
                 self.global_statistics.record_latency(self.wid, decode_time, start_send_at, end_send_at, pixels, bytecount)
-        else:
-            #something failed client-side, so we can't rely on the delta being available
-            self.last_pixmap_data = None
         if self._damage_delayed is not None and self._damage_delayed_expired:
             self.idle_add(self.may_send_delayed)
         if not self._damage_delayed:
             self.soft_expired = 0
 
+    def client_decode_error(self, error):
+        #something failed client-side, so we can't rely on the delta being available
+        self.last_pixmap_data = None
+
+
     def make_data_packet(self, damage_time, process_damage_time, wid, image, coding, sequence, options):
         """
             Picture encoding - non-UI thread.
@@ -1302,10 +1319,12 @@ class WindowSource(object):
         #if client supports delta pre-compression for this encoding, use it if we can:
         delta = -1
         store = -1
-        if DELTA and not (self._mmap and self._mmap_size>0) and (coding in self.supports_delta) and self.min_delta_size<image.get_size()<self.max_delta_size:
-            #we need to copy the pixels because some delta encodings
-            #will modify the pixel array in-place!
-            dpixels = image.get_pixels()[:]
+        isize = image.get_width() * image.get_height()
+        if DELTA and w>2 and h>2 and not (self._mmap and self._mmap_size>0) and (coding in self.supports_delta) and self.min_delta_size<isize<self.max_delta_size:
+            dpixels = image.get_pixels()
+            #hack note: the '[:]' slicing does not make a copy when dealing with a memoryview
+            #but it does when dealing with strings! (and so we only make one copy no matter what here)
+            dpixels = memoryview_to_bytes(dpixels[:])
             store = sequence
             lpd = self.last_pixmap_data
             if lpd is not None:
