@@ -9,6 +9,7 @@
 Everyone else should just use prop_set/prop_get with nice clean Python calling
 conventions, and if you need more (un)marshalling smarts, add them here."""
 
+import binascii
 import struct
 import gtk.gdk
 import cairo
@@ -186,6 +187,9 @@ class NetWMStrut(object):
         # will be only length 4 instead of 12, but _force_length will zero-pad
         # and _NET_WM_STRUT is *defined* as a _NET_WM_STRUT_PARTIAL where the
         # extra fields are zero... so it all works out.
+        if len(data)==16:
+            self.left, self.right, self.top, self.bottom = struct.unpack("=IIII", data)
+            return
         data = _force_length("_NET_WM_STRUT or _NET_WM_STRUT_PARTIAL", data, 4 * 12)
         (self.left, self.right, self.top, self.bottom,
          self.left_start_y, self.left_end_y,
@@ -195,11 +199,23 @@ class NetWMStrut(object):
          ) = struct.unpack("=" + "I" * 12, data)
 
     def __str__(self):
-        return "NetWMStrut(%s)" % str(self.left, self.right, self.top, self.bottom,
-                                     self.left_start_y, self.left_end_y,
-                                     self.right_start_y, self.right_end_y,
-                                     self.top_start_x, self.top_end_x,
-                                     self.bottom_start_x, self.bottom_stop_x)
+        return "NetWMStrut(%s)" % self.__dict__
+
+
+class MotifWMHints(object):
+    def __init__(self, disp, data):
+        #some applications use the wrong size (ie: blender uses 16) so pad it:
+        pdata = _force_length("_MOTIF_WM_HINTS", data, 20, 16)
+        self.flags, self.functions, self.decorations, self.input_mode, self.status = \
+            struct.unpack("=IIIiI", pdata)
+        log("MotifWMHints(%s)=%s", binascii.hexlify(data), self)
+
+    def __str__(self):
+        return "MotifWMHints(%s)" % {"flags"        : self.flags,
+                                     "functions"    : self.functions,
+                                     "decorations"  : self.decorations,
+                                     "input_mode"   : self.input_mode,
+                                     "status"       : self.status}
 
 
 def _read_image(disp, stream):
@@ -285,6 +301,10 @@ _prop_types = {
              lambda disp, a: struct.pack("@I", get_xatom(a)),
               _get_atom,
              ""),
+    "state": ((int, long), "WM_STATE", 32,
+            lambda disp, c: struct.pack("=I", c),
+            lambda disp, d: struct.unpack("=I", d)[0],
+            ""),
     "u32": ((int, long), "CARDINAL", 32,
             lambda disp, c: struct.pack("=I", c),
             lambda disp, d: struct.unpack("=I", d)[0],
@@ -309,6 +329,8 @@ _prop_types = {
               unsupported, NetWMStrut, None),
     "strut-partial": (NetWMStrut, "CARDINAL", 32,
                       unsupported, NetWMStrut, None),
+    "motif-hints": (MotifWMHints, "_MOTIF_WM_HINTS", 32,
+              unsupported, MotifWMHints, None),
     "icon": (cairo.ImageSurface, "CARDINAL", 32,
              unsupported, NetWMIcons, None),
     "xsettings-settings": (tuple, "_XSETTINGS_SETTINGS", 8,
