@@ -43,7 +43,7 @@ NotifyPointerRoot   = constants["NotifyPointerRoot"]
 NotifyDetailNone    = constants["NotifyDetailNone"]
 
 
-def wm_check(display, upgrading=False):
+def wm_check(display, wm_name, upgrading=False):
     #there should only be one screen... but let's check all of them
     for i in range(display.get_n_screens()):
         screen = display.get_screen(i)
@@ -67,7 +67,7 @@ def wm_check(display, upgrading=False):
                 name = prop_get(ewmh_wm, "_NET_WM_NAME", "utf8", ignore_errors=False, raise_xerrors=False)
             except:
                 name = None
-            if upgrading and name and name==XPRA_NET_WM_NAME:
+            if upgrading and name and name==wm_name:
                 log.info("found previous Xpra instance")
             else:
                 log.warn("Warning: found an existing window manager on screen %s using window %#x: %s", i, ewmh_wm.xid, name or "unknown")
@@ -156,6 +156,10 @@ class Wm(gobject.GObject):
         # _NET_WM_STATE_ABOVE,
         # _NET_WM_STATE_BELOW,
 
+        "_MOTIF_WM_HINTS",
+        "_MOTIF_WM_INFO",
+        "_NET_WM_MOVERESIZE",
+
         # Not at all yet:
         #"_NET_REQUEST_FRAME_EXTENTS",
         #"_NET_CLOSE_WINDOW",
@@ -193,7 +197,7 @@ class Wm(gobject.GObject):
         "xpra-xkb-event": one_arg_signal,
         }
 
-    def __init__(self, replace_other_wm, display=None):
+    def __init__(self, replace_other_wm, wm_name, display=None):
         gobject.GObject.__init__(self)
 
         if display is None:
@@ -201,6 +205,7 @@ class Wm(gobject.GObject):
         self._display = display
         self._alt_display = gtk.gdk.Display(self._display.get_name())
         self._root = self._display.get_default_screen().get_root_window()
+        self._wm_name = wm_name
         self._ewmh_window = None
 
         self._windows = {}
@@ -431,11 +436,11 @@ class Wm(gobject.GObject):
                                            window_type=gtk.gdk.WINDOW_TOPLEVEL,
                                            event_mask=0, # event mask
                                            wclass=gtk.gdk.INPUT_ONLY,
-                                           title=XPRA_NET_WM_NAME)
+                                           title=self._wm_name)
         prop_set(self._ewmh_window, "_NET_SUPPORTING_WM_CHECK",
                  "window", self._ewmh_window)
-        self.root_set("_NET_SUPPORTING_WM_CHECK",
-                 "window", self._ewmh_window)
+        self.root_set("_NET_SUPPORTING_WM_CHECK", "window", self._ewmh_window)
+        self.root_set("_NET_WM_NAME", "utf8", self._wm_name.decode("utf8"))
 
     # Other global actions:
 
@@ -446,7 +451,7 @@ class Wm(gobject.GObject):
     def get_net_wm_name(self):
         try:
             return prop_get(self._ewmh_window, "_NET_WM_NAME", "utf8", ignore_errors=False, raise_xerrors=False)
-        except Exception as e:
+        except Exception, e:
             log.error("error querying _NET_WM_NAME: %s", e)
 
 gobject.type_register(Wm)
